@@ -1,4 +1,5 @@
-/*
+/* pk-backend-aptcc.cpp
+ *
  * Copyright (C) 2007-2008 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2009-2012 Daniel Nicoletti <dantti12@gmail.com>
  *
@@ -29,7 +30,7 @@
 #include <apt-pkg/init.h>
 
 #include "apt-intf.h"
-#include "AptCacheFile.h"
+#include "apt-cache-file.h"
 #include "apt-messages.h"
 #include "acqpkitstatus.h"
 #include "apt-sourceslist.h"
@@ -59,8 +60,8 @@ const gchar* pk_backend_get_author(PkBackend *backend)
 gboolean
 pk_backend_supports_parallelization (PkBackend *backend)
 {
-	// we need to set this to TRUE as soon as the parallelization work is completed!
-	return FALSE;
+    // we need to set this to TRUE as soon as the parallelization work is completed!
+    return FALSE;
 }
 
 /**
@@ -90,7 +91,7 @@ void pk_backend_initialize(GKeyFile *conf, PkBackend *backend)
     }
 
     spawn = pk_backend_spawn_new(conf);
-//     pk_backend_spawn_set_job(spawn, backend);
+    //     pk_backend_spawn_set_job(spawn, backend);
     pk_backend_spawn_set_name(spawn, "aptcc");
 }
 
@@ -161,7 +162,9 @@ PkBitfield pk_backend_get_filters(PkBackend *backend)
  */
 gchar** pk_backend_get_mime_types(PkBackend *backend)
 {
-    const gchar *mime_types[] = { "application/x-deb", NULL };
+    const gchar *mime_types[] = { "application/vnd.debian.binary-package",
+                                  "application/x-deb",
+                                  NULL };
     return g_strdupv ((gchar **) mime_types);
 }
 
@@ -175,14 +178,6 @@ void pk_backend_start_job(PkBackend *backend, PkBackendJob *job)
     pk_backend_job_set_user_data(job, apt);
 }
 
-/**
- * pk_backend_reset_job:
- */
-void pk_backend_reset_job(PkBackend *backend, PkBackendJob *job)
-{
-    // To keep the cache alive we just need to be defined...
-}
-
 /**
  * pk_backend_stop_job:
  */
@@ -229,7 +224,6 @@ static void backend_depends_on_or_requires_thread(PkBackendJob *job, GVariant *p
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
-        apt->emitFinished();
         return;
     }
 
@@ -245,7 +239,6 @@ static void backend_depends_on_or_requires_thread(PkBackendJob *job, GVariant *p
                                       PK_ERROR_ENUM_PACKAGE_ID_INVALID,
                                       "%s",
                                       pi);
-            apt->emitFinished();
             return;
         }
 
@@ -255,7 +248,6 @@ static void backend_depends_on_or_requires_thread(PkBackendJob *job, GVariant *p
                                       PK_ERROR_ENUM_PACKAGE_NOT_FOUND,
                                       "Couldn't find package %s",
                                       pi);
-            apt->emitFinished();
             return;
         }
 
@@ -266,17 +258,15 @@ static void backend_depends_on_or_requires_thread(PkBackendJob *job, GVariant *p
         }
     }
 
-    // It's faster to emmit the packages here than in the matching part
+    // It's faster to emit the packages here than in the matching part
     apt->emitPackages(output, filters);
-
-    apt->emitFinished();
 }
 
 /**
  * pk_backend_depends_on:
  */
 void pk_backend_depends_on(PkBackend *backend, PkBackendJob *job, PkBitfield filters,
-                            gchar **package_ids, gboolean recursive)
+                           gchar **package_ids, gboolean recursive)
 {
     pk_backend_job_thread_create(job, backend_depends_on_or_requires_thread, NULL, NULL);
 }
@@ -285,10 +275,10 @@ void pk_backend_depends_on(PkBackend *backend, PkBackendJob *job, PkBitfield fil
  * pk_backend_required_by:
  */
 void pk_backend_required_by(PkBackend *backend,
-                             PkBackendJob *job,
-                             PkBitfield filters,
-                             gchar **package_ids,
-                             gboolean recursive)
+                            PkBackendJob *job,
+                            PkBitfield filters,
+                            gchar **package_ids,
+                            gboolean recursive)
 {
     pk_backend_job_thread_create(job, backend_depends_on_or_requires_thread, NULL, NULL);
 }
@@ -312,7 +302,6 @@ static void backend_get_files_thread(PkBackendJob *job, GVariant *params, gpoint
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
-        apt->emitFinished();
         return;
     }
 
@@ -320,8 +309,6 @@ static void backend_get_files_thread(PkBackendJob *job, GVariant *params, gpoint
         pk_backend_job_error_code(job,
                                   PK_ERROR_ENUM_PACKAGE_ID_INVALID,
                                   "Invalid package id");
-        pk_backend_job_finished(job);
-        apt->emitFinished();
         return;
     }
 
@@ -333,7 +320,6 @@ static void backend_get_files_thread(PkBackendJob *job, GVariant *params, gpoint
                                       PK_ERROR_ENUM_PACKAGE_ID_INVALID,
                                       "%s",
                                       pi);
-            apt->emitFinished();
             return;
         }
 
@@ -343,14 +329,11 @@ static void backend_get_files_thread(PkBackendJob *job, GVariant *params, gpoint
                                       PK_ERROR_ENUM_PACKAGE_NOT_FOUND,
                                       "Couldn't find package %s",
                                       pi);
-            apt->emitFinished();
             return;
         }
 
         apt->emitPackageFiles(pi);
     }
-
-    apt->emitFinished();
 }
 
 /**
@@ -373,7 +356,6 @@ static void backend_get_details_thread(PkBackendJob *job, GVariant *params, gpoi
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
     if (!apt->init()) {
         g_debug ("Failed to create apt cache");
-        apt->emitFinished();
         return;
     }
 
@@ -381,8 +363,6 @@ static void backend_get_details_thread(PkBackendJob *job, GVariant *params, gpoi
         pk_backend_job_error_code(job,
                                   PK_ERROR_ENUM_PACKAGE_ID_INVALID,
                                   "Invalid package id");
-        pk_backend_job_finished(job);
-        apt->emitFinished();
         return;
     }
 
@@ -394,8 +374,6 @@ static void backend_get_details_thread(PkBackendJob *job, GVariant *params, gpoi
     } else {
         apt->emitDetails(pkgs);
     }
-
-    apt->emitFinished();
 }
 
 /**
@@ -424,7 +402,6 @@ static void backend_get_updates_thread(PkBackendJob *job, GVariant *params, gpoi
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
-        apt->emitFinished();
         return;
     }
 
@@ -436,8 +413,6 @@ static void backend_get_updates_thread(PkBackendJob *job, GVariant *params, gpoi
 
     apt->emitUpdates(updates, filters);
     apt->emitPackages(blocked, filters, PK_INFO_ENUM_BLOCKED);
-
-    apt->emitFinished();
 }
 
 /**
@@ -466,7 +441,6 @@ static void backend_what_provides_thread(PkBackendJob *job, GVariant *params, gp
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
         g_strfreev(values);
-        apt->emitFinished();
         return;
     }
 
@@ -479,8 +453,6 @@ static void backend_what_provides_thread(PkBackendJob *job, GVariant *params, gp
 
     // It's faster to emit the packages here rather than in the matching part
     apt->emitPackages(output, filters);
-
-    apt->emitFinished();
 }
 
 /**
@@ -512,7 +484,6 @@ static void pk_backend_download_packages_thread(PkBackendJob *job, GVariant *par
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
-        apt->emitFinished();
         return;
     }
 
@@ -523,8 +494,7 @@ static void pk_backend_download_packages_thread(PkBackendJob *job, GVariant *par
         AcqPackageKitStatus Stat(apt, job);
 
         // get a fetcher
-        pkgAcquire fetcher;
-        fetcher.Setup(&Stat);
+        pkgAcquire fetcher(&Stat);
         gchar *pi;
 
         // TODO this might be useful when the item is in the cache
@@ -551,10 +521,9 @@ static void pk_backend_download_packages_thread(PkBackendJob *job, GVariant *par
             pi = package_ids[i];
             if (pk_package_id_check(pi) == false) {
                 pk_backend_job_error_code(job,
-                                        PK_ERROR_ENUM_PACKAGE_ID_INVALID,
-                                        "%s",
-                                        pi);
-                apt->emitFinished();
+                                          PK_ERROR_ENUM_PACKAGE_ID_INVALID,
+                                          "%s",
+                                          pi);
                 return;
             }
 
@@ -570,17 +539,16 @@ static void pk_backend_download_packages_thread(PkBackendJob *job, GVariant *par
             } else {
                 if(!ver.Downloadable()) {
                     _error->Error("No downloadable files for %s,"
-                                "perhaps it is a local or obsolete" "package?",
-                                pi);
+                                  "perhaps it is a local or obsolete" "package?",
+                                  pi);
                     continue;
                 }
 
                 string storeFileName;
                 if (!apt->getArchive(&fetcher,
-                                    ver,
-                                    directory,
-                                    storeFileName)) {
-                    apt->emitFinished();
+                                     ver,
+                                     directory,
+                                     storeFileName)) {
                     return;
                 }
 
@@ -596,7 +564,6 @@ static void pk_backend_download_packages_thread(PkBackendJob *job, GVariant *par
                 && apt->cancelled() == false) {
             // We failed and we did not cancel
             show_errors(job, PK_ERROR_ENUM_PACKAGE_DOWNLOAD_FAILED);
-            apt->emitFinished();
             return;
         }
 
@@ -605,8 +572,6 @@ static void pk_backend_download_packages_thread(PkBackendJob *job, GVariant *par
                                   PK_ERROR_ENUM_NO_NETWORK,
                                   "Cannot download packages whilst offline");
     }
-
-    apt->emitFinished();
 }
 
 /**
@@ -630,7 +595,6 @@ static void pk_backend_refresh_cache_thread(PkBackendJob *job, GVariant *params,
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
-        apt->emitFinished();
         return;
     }
     
@@ -646,8 +610,6 @@ static void pk_backend_refresh_cache_thread(PkBackendJob *job, GVariant *params,
                                   PK_ERROR_ENUM_NO_NETWORK,
                                   "Cannot refresh cache whilst offline");
     }
-
-    apt->emitFinished();
 }
 
 /**
@@ -671,16 +633,13 @@ static void pk_backend_resolve_thread(PkBackendJob *job, GVariant *params, gpoin
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
-        apt->emitFinished();
         return;
     }
 
     PkgList pkgs = apt->resolvePackageIds(search);
 
-    // It's faster to emmit the packages here rather than in the matching part
+    // It's faster to emit the packages here rather than in the matching part
     apt->emitPackages(pkgs, filters);
-
-    apt->emitFinished();
 }
 
 /**
@@ -707,7 +666,6 @@ static void pk_backend_search_files_thread(PkBackendJob *job, GVariant *params,
     if (!pk_bitfield_contain(filters, PK_FILTER_ENUM_NOT_INSTALLED)) {
         if (!apt->init()) {
             g_debug("Failed to create apt cache");
-            apt->emitFinished();
             return;
         }
 
@@ -718,8 +676,6 @@ static void pk_backend_search_files_thread(PkBackendJob *job, GVariant *params,
         // It's faster to emit the packages here rather than in the matching part
         apt->emitPackages(output, filters);
     }
-
-    apt->emitFinished();
 }
 
 /**
@@ -742,18 +698,15 @@ static void backend_search_groups_thread(PkBackendJob *job, GVariant *params, gp
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
-        apt->emitFinished();
         return;
     }
 
-    // It's faster to emmit the packages here rather than in the matching part
+    // It's faster to emit the packages here rather than in the matching part
     PkgList output;
     output = apt->getPackagesFromGroup(search);
     apt->emitPackages(output, filters);
 
     pk_backend_job_set_percentage(job, 100);
-
-    apt->emitFinished();
 }
 
 /**
@@ -780,13 +733,11 @@ static void backend_search_package_thread(PkBackendJob *job, GVariant *params, g
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
         g_free(search);
-        apt->emitFinished();
         return;
     }
 
     if (_error->PendingError() == true) {
         g_free(search);
-        apt->emitFinished();
         return;
     }
 
@@ -803,12 +754,10 @@ static void backend_search_package_thread(PkBackendJob *job, GVariant *params, g
     }
     g_free(search);
 
-    // It's faster to emmit the packages here than in the matching part
+    // It's faster to emit the packages here than in the matching part
     apt->emitPackages(output, filters);
 
     pk_backend_job_set_percentage(job, 100);
-
-    apt->emitFinished();
 }
 
 /**
@@ -881,7 +830,6 @@ static void backend_manage_packages_thread(PkBackendJob *job, GVariant *params,
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
-        apt->emitFinished();
         return;
     }
 
@@ -894,15 +842,13 @@ static void backend_manage_packages_thread(PkBackendJob *job, GVariant *params,
         // GDebi can not install more than one package at time
         if (g_strv_length(full_paths) > 1) {
             pk_backend_job_error_code(job,
-                                  PK_ERROR_ENUM_NOT_SUPPORTED,
-                                  "The backend can only proccess one file at time.");
-            apt->emitFinished();
+                                      PK_ERROR_ENUM_NOT_SUPPORTED,
+                                      "The backend can only process one file at time.");
             return;
         }
 
         // get the list of packages to install
         if (!apt->markFileForInstall(full_paths[0], installPkgs, removePkgs)) {
-            apt->emitFinished();
             return;
         }
 
@@ -921,7 +867,6 @@ static void backend_manage_packages_thread(PkBackendJob *job, GVariant *params,
             pk_backend_job_error_code(job,
                                       PK_ERROR_ENUM_PACKAGE_NOT_FOUND,
                                       "Could not find package(s)");
-            apt->emitFinished();
             return;
         }
     }
@@ -931,14 +876,13 @@ static void backend_manage_packages_thread(PkBackendJob *job, GVariant *params,
     ret = apt->runTransaction(installPkgs,
                               removePkgs,
                               fileInstall, // Mark newly installed packages as auto-installed
-                                           // (they're dependencies of the new local package)
+                              // (they're dependencies of the new local package)
                               fixBroken,
                               transaction_flags,
                               autoremove);
     if (!ret) {
         // Print transaction errors
-        g_debug("AptIntf::runTransaction() failed: ", _error->PendingError());
-        apt->emitFinished();
+        g_debug("AptIntf::runTransaction() failed: %i", _error->PendingError());
         return;
     }
 
@@ -947,20 +891,16 @@ static void backend_manage_packages_thread(PkBackendJob *job, GVariant *params,
         gchar *path;
         for (uint i = 0; i < g_strv_length(full_paths); ++i) {
             if (apt->cancelled()) {
-                apt->emitFinished();
                 break;
             }
 
             path = full_paths[i];
             if (!apt->installFile(path, simulate)) {
                 cout << "Installation of DEB file " << path << " failed." << endl;
-                apt->emitFinished();
                 return;
             }
         }
     }
-
-    apt->emitFinished();
 }
 
 /**
@@ -1058,7 +998,6 @@ static void backend_repo_manager_thread(PkBackendJob *job, GVariant *params, gpo
     if (_lst.ReadVendors() == false) {
         _error->Error("Cannot read vendors.list file");
         show_errors(job, PK_ERROR_ENUM_FAILED_CONFIG_PARSING);
-        apt->emitFinished();
         return;
     }
 
@@ -1074,10 +1013,10 @@ static void backend_repo_manager_thread(PkBackendJob *job, GVariant *params, gpo
 
         if (role == PK_ROLE_ENUM_GET_REPO_LIST) {
             if (pk_bitfield_contain(filters, PK_FILTER_ENUM_NOT_DEVELOPMENT) &&
-                ((*it)->Type & SourcesList::DebSrc ||
-                 (*it)->Type & SourcesList::RpmSrc ||
-                 (*it)->Type & SourcesList::RpmSrcDir ||
-                 (*it)->Type & SourcesList::RepomdSrc)) {
+                    ((*it)->Type & SourcesList::DebSrc ||
+                     (*it)->Type & SourcesList::RpmSrc ||
+                     (*it)->Type & SourcesList::RpmSrcDir ||
+                     (*it)->Type & SourcesList::RepomdSrc)) {
                 continue;
             }
 
@@ -1106,7 +1045,6 @@ static void backend_repo_manager_thread(PkBackendJob *job, GVariant *params, gpo
                     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
                     if (!apt->init()) {
                         g_debug("Failed to create apt cache");
-                        apt->emitFinished();
                         return;
                     }
 
@@ -1115,15 +1053,14 @@ static void backend_repo_manager_thread(PkBackendJob *job, GVariant *params, gpo
                         // Install/Update/Remove packages, or just simulate
                         bool ret;
                         ret = apt->runTransaction(PkgList(),
-                                                removePkgs,
-                                                false,
-                                                false,
-                                                transaction_flags,
-                                                false);
+                                                  removePkgs,
+                                                  false,
+                                                  false,
+                                                  transaction_flags,
+                                                  false);
                         if (!ret) {
                             // Print transaction errors
-                            g_debug("AptIntf::runTransaction() failed: ", _error->PendingError());
-                            apt->emitFinished();
+                            g_debug("AptIntf::runTransaction() failed: %i", _error->PendingError());
                             return;
                         }
                     }
@@ -1147,12 +1084,10 @@ static void backend_repo_manager_thread(PkBackendJob *job, GVariant *params, gpo
     }
 
     if ((role == PK_ROLE_ENUM_REPO_ENABLE || role == PK_ROLE_ENUM_REPO_REMOVE) &&
-        !found) {
+            !found) {
         _error->Error("Could not found the repository");
         show_errors(job, PK_ERROR_ENUM_REPO_NOT_AVAILABLE);
     }
-
-    apt->emitFinished();
 }
 
 /**
@@ -1194,17 +1129,14 @@ static void backend_get_packages_thread(PkBackendJob *job, GVariant *params, gpo
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
     if (!apt->init()) {
         g_debug("Failed to create apt cache");
-        apt->emitFinished();
         return;
     }
 
     PkgList output;
     output = apt->getPackages();
 
-    // It's faster to emmit the packages rather here than in the matching part
+    // It's faster to emit the packages rather here than in the matching part
     apt->emitPackages(output, filters);
-
-    apt->emitFinished();
 }
 
 /**
