@@ -36,6 +36,7 @@
 #include <packagekit-glib2/pk-common.h>
 #include <packagekit-glib2/pk-control.h>
 #include <packagekit-glib2/pk-version.h>
+#include <packagekit-glib2/pk-enum-types.h>
 
 static void     pk_control_finalize	(GObject     *object);
 
@@ -134,14 +135,7 @@ typedef struct {
  *
  * Since: 0.5.2
  **/
-GQuark
-pk_control_error_quark (void)
-{
-	static GQuark quark = 0;
-	if (!quark)
-		quark = g_quark_from_static_string ("pk_control_error");
-	return quark;
-}
+G_DEFINE_QUARK (pk-control-error-quark, pk_control_error)
 
 /**
  * pk_control_fixup_dbus_error:
@@ -189,7 +183,6 @@ pk_control_set_property_value (PkControl *control,
 			       GVariant *value)
 {
 	const gchar *tmp_str;
-	gchar **tmp_strv = NULL;
 	gboolean tmp_bool;
 	guint tmp_uint;
 	PkBitfield tmp_bitfield;
@@ -246,13 +239,14 @@ pk_control_set_property_value (PkControl *control,
 		return;
 	}
 	if (g_strcmp0 (key, "MimeTypes") == 0) {
+		g_autofree gchar **tmp_strv = NULL;
 		tmp_strv = (gchar **) g_variant_get_strv (value, NULL);
 		if (_g_strvcmp0 (control->priv->mime_types, tmp_strv))
-			goto out;
+			return;
 		g_strfreev (control->priv->mime_types);
 		control->priv->mime_types = g_strdupv (tmp_strv);
 		g_object_notify (G_OBJECT(control), "mime-types");
-		goto out;
+		return;
 	}
 	if (g_strcmp0 (key, "Roles") == 0) {
 		tmp_bitfield = g_variant_get_uint64 (value);
@@ -315,8 +309,6 @@ pk_control_set_property_value (PkControl *control,
 		return;
 	}
 	g_warning ("unhandled property '%s'", key);
-out:
-	g_free (tmp_strv);
 }
 
 /**
@@ -354,8 +346,8 @@ pk_control_signal_cb (GDBusProxy *proxy,
 		      gpointer user_data)
 {
 	const gchar **ids_tmp = NULL;
-	gchar **ids = NULL;
 	PkControl *control = PK_CONTROL (user_data);
+	g_auto(GStrv) ids = NULL;
 
 	if (g_strcmp0 (signal_name, "TransactionListChanged") == 0) {
 		g_variant_get (parameters, "(^a&s)", &ids_tmp);
@@ -372,21 +364,18 @@ pk_control_signal_cb (GDBusProxy *proxy,
 	if (g_strcmp0 (signal_name, "UpdatesChanged") == 0) {
 		g_debug ("emit updates-changed");
 		g_signal_emit (control, signals[SIGNAL_UPDATES_CHANGED], 0);
-		goto out;
+		return;
 	}
 	if (g_strcmp0 (signal_name, "RepoListChanged") == 0) {
 		g_debug ("emit repo-list-changed");
 		g_signal_emit (control, signals[SIGNAL_REPO_LIST_CHANGED], 0);
-		goto out;
+		return;
 	}
 	if (g_strcmp0 (signal_name, "RestartSchedule") == 0) {
 		g_debug ("emit restart-schedule");
 		g_signal_emit (control, signals[SIGNAL_RESTART_SCHEDULE], 0);
-		goto out;
+		return;
 	}
-out:
-	g_free (ids_tmp);
-	g_strfreev (ids);
 }
 
 /**
@@ -395,19 +384,17 @@ out:
 static void
 pk_control_proxy_connect (PkControlState *state)
 {
-	gchar **props = NULL;
 	guint i;
-	GVariant *value_tmp;
+	g_auto(GStrv) props = NULL;
 
 	/* coldplug properties */
 	props = g_dbus_proxy_get_cached_property_names (state->proxy);
 	for (i = 0; props != NULL && props[i] != NULL; i++) {
-		value_tmp = g_dbus_proxy_get_cached_property (state->proxy,
-							      props[i]);
+		g_autoptr(GVariant) value_tmp = NULL;
+		value_tmp = g_dbus_proxy_get_cached_property (state->proxy, props[i]);
 		pk_control_set_property_value (state->control,
 					       props[i],
 					       value_tmp);
-		g_variant_unref (value_tmp);
 	}
 
 	/* connect up signals */
@@ -421,8 +408,6 @@ pk_control_proxy_connect (PkControlState *state)
 	/* if we have no generic system wide proxy, then use this */
 	if (state->control->priv->proxy == NULL)
 		state->control->priv->proxy = g_object_ref (state->proxy);
-
-	g_strfreev (props);
 }
 
 /**********************************************************************/
@@ -470,10 +455,10 @@ pk_control_get_tid_cb (GObject *source_object,
 		       GAsyncResult *res,
 		       gpointer user_data)
 {
-	GError *error = NULL;
-	GVariant *value;
 	GDBusProxy *proxy = G_DBUS_PROXY (source_object);
 	PkControlState *state = (PkControlState *) user_data;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GVariant) value = NULL;
 
 	/* get the result */
 	value = g_dbus_proxy_call_finish (proxy, res, &error);
@@ -481,8 +466,7 @@ pk_control_get_tid_cb (GObject *source_object,
 		/* fix up the D-Bus error */
 		pk_control_fixup_dbus_error (error);
 		pk_control_get_tid_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* save results */
@@ -490,9 +474,6 @@ pk_control_get_tid_cb (GObject *source_object,
 
 	/* we're done */
 	pk_control_get_tid_state_finish (state, NULL);
-out:
-	if (value != NULL)
-		g_variant_unref (value);
 }
 
 /**
@@ -519,13 +500,12 @@ pk_control_get_tid_proxy_cb (GObject *source_object,
 			     GAsyncResult *res,
 			     gpointer user_data)
 {
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	PkControlState *state = (PkControlState *) user_data;
 
 	state->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 	if (state->proxy == NULL) {
 		pk_control_get_tid_state_finish (state, error);
-		g_error_free (error);
 		return;
 	}
 	pk_control_proxy_connect (state);
@@ -549,9 +529,9 @@ pk_control_get_tid_async (PkControl *control,
 			  GAsyncReadyCallback callback,
 			  gpointer user_data)
 {
-	GSimpleAsyncResult *res;
 	PkControlState *state;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GSimpleAsyncResult) res = NULL;
 
 	g_return_if_fail (PK_IS_CONTROL (control));
 	g_return_if_fail (callback != NULL);
@@ -573,8 +553,7 @@ pk_control_get_tid_async (PkControl *control,
 	if (cancellable != NULL &&
 	    g_cancellable_set_error_if_cancelled (cancellable, &error)) {
 		pk_control_get_tid_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* skip straight to the D-Bus method if already connection */
@@ -594,8 +573,6 @@ pk_control_get_tid_async (PkControl *control,
 
 	/* track state */
 	g_ptr_array_add (control->priv->calls, state);
-out:
-	g_object_unref (res);
 }
 
 /**
@@ -678,17 +655,16 @@ pk_control_suggest_daemon_quit_cb (GObject *source_object,
 				   GAsyncResult *res,
 				   gpointer user_data)
 {
-	GError *error = NULL;
-	GVariant *value;
 	GDBusProxy *proxy = G_DBUS_PROXY (source_object);
 	PkControlState *state = (PkControlState *) user_data;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GVariant) value = NULL;
 
 	/* get the result */
 	value = g_dbus_proxy_call_finish (proxy, res, &error);
 	if (value == NULL) {
 		pk_control_suggest_daemon_quit_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* save data */
@@ -696,9 +672,6 @@ pk_control_suggest_daemon_quit_cb (GObject *source_object,
 
 	/* we're done */
 	pk_control_suggest_daemon_quit_state_finish (state, NULL);
-out:
-	if (value != NULL)
-		g_variant_unref (value);
 }
 
 /**
@@ -725,13 +698,12 @@ pk_control_suggest_daemon_quit_proxy_cb (GObject *source_object,
 					 GAsyncResult *res,
 					 gpointer user_data)
 {
-	GError *error = NULL;
 	PkControlState *state = (PkControlState *) user_data;
+	g_autoptr(GError) error = NULL;
 
 	state->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 	if (state->proxy == NULL) {
 		pk_control_suggest_daemon_quit_state_finish (state, error);
-		g_error_free (error);
 		return;
 	}
 	pk_control_proxy_connect (state);
@@ -755,9 +727,9 @@ pk_control_suggest_daemon_quit_async (PkControl *control,
 				      GAsyncReadyCallback callback,
 				      gpointer user_data)
 {
-	GSimpleAsyncResult *res;
 	PkControlState *state;
-	GError *error = NULL;
+	g_autoptr(GSimpleAsyncResult) res = NULL;
+	g_autoptr(GError) error = NULL;
 
 	g_return_if_fail (PK_IS_CONTROL (control));
 	g_return_if_fail (callback != NULL);
@@ -779,8 +751,7 @@ pk_control_suggest_daemon_quit_async (PkControl *control,
 	if (cancellable != NULL &&
 	    g_cancellable_set_error_if_cancelled (cancellable, &error)) {
 		pk_control_suggest_daemon_quit_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* skip straight to the D-Bus method if already connection */
@@ -800,8 +771,6 @@ pk_control_suggest_daemon_quit_async (PkControl *control,
 
 	/* track state */
 	g_ptr_array_add (control->priv->calls, state);
-out:
-	g_object_unref (res);
 }
 
 /**
@@ -882,10 +851,10 @@ pk_control_get_daemon_state_cb (GObject *source_object,
 				   GAsyncResult *res,
 				   gpointer user_data)
 {
-	GError *error = NULL;
-	GVariant *value;
 	GDBusProxy *proxy = G_DBUS_PROXY (source_object);
 	PkControlState *state = (PkControlState *) user_data;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GVariant) value = NULL;
 
 	/* get the result */
 	value = g_dbus_proxy_call_finish (proxy, res, &error);
@@ -893,8 +862,7 @@ pk_control_get_daemon_state_cb (GObject *source_object,
 		/* fix up the D-Bus error */
 		pk_control_fixup_dbus_error (error);
 		pk_control_get_daemon_state_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* save results */
@@ -902,9 +870,6 @@ pk_control_get_daemon_state_cb (GObject *source_object,
 
 	/* we're done */
 	pk_control_get_daemon_state_state_finish (state, NULL);
-out:
-	if (value != NULL)
-		g_variant_unref (value);
 }
 
 /**
@@ -931,13 +896,12 @@ pk_control_get_daemon_state_proxy_cb (GObject *source_object,
 			     GAsyncResult *res,
 			     gpointer user_data)
 {
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	PkControlState *state = (PkControlState *) user_data;
 
 	state->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 	if (state->proxy == NULL) {
 		pk_control_get_daemon_state_state_finish (state, error);
-		g_error_free (error);
 		return;
 	}
 	pk_control_proxy_connect (state);
@@ -961,9 +925,9 @@ pk_control_get_daemon_state_async (PkControl *control,
 				   GAsyncReadyCallback callback,
 				   gpointer user_data)
 {
-	GSimpleAsyncResult *res;
 	PkControlState *state;
-	GError *error = NULL;
+	g_autoptr(GSimpleAsyncResult) res = NULL;
+	g_autoptr(GError) error = NULL;
 
 	g_return_if_fail (PK_IS_CONTROL (control));
 	g_return_if_fail (callback != NULL);
@@ -985,8 +949,7 @@ pk_control_get_daemon_state_async (PkControl *control,
 	if (cancellable != NULL &&
 	    g_cancellable_set_error_if_cancelled (cancellable, &error)) {
 		pk_control_get_daemon_state_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* skip straight to the D-Bus method if already connection */
@@ -1006,8 +969,6 @@ pk_control_get_daemon_state_async (PkControl *control,
 
 	/* track state */
 	g_ptr_array_add (control->priv->calls, state);
-out:
-	g_object_unref (res);
 }
 
 /**
@@ -1091,18 +1052,17 @@ pk_control_set_proxy_cb (GObject *source_object,
 				   GAsyncResult *res,
 				   gpointer user_data)
 {
-	GError *error = NULL;
-	GVariant *value;
 	GDBusProxy *proxy = G_DBUS_PROXY (source_object);
 	PkControlState *state = (PkControlState *) user_data;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GVariant) value = NULL;
 
 	/* get the result */
 	value = g_dbus_proxy_call_finish (proxy, res, &error);
 	if (value == NULL) {
 		g_warning ("failed to set proxy: %s", error->message);
 		pk_control_set_proxy_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* save data */
@@ -1110,9 +1070,6 @@ pk_control_set_proxy_cb (GObject *source_object,
 
 	/* we're done */
 	pk_control_set_proxy_state_finish (state, NULL);
-out:
-	if (value != NULL)
-		g_variant_unref (value);
 }
 
 /**
@@ -1139,13 +1096,12 @@ pk_control_set_proxy_proxy_cb (GObject *source_object,
 			     GAsyncResult *res,
 			     gpointer user_data)
 {
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	PkControlState *state = (PkControlState *) user_data;
 
 	state->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 	if (state->proxy == NULL) {
 		pk_control_set_proxy_state_finish (state, error);
-		g_error_free (error);
 		return;
 	}
 	pk_control_proxy_connect (state);
@@ -1181,9 +1137,9 @@ pk_control_set_proxy2_async (PkControl *control,
 			     GAsyncReadyCallback callback,
 			     gpointer user_data)
 {
-	GSimpleAsyncResult *res;
 	PkControlState *state;
-	GError *error = NULL;
+	g_autoptr(GSimpleAsyncResult) res = NULL;
+	g_autoptr(GError) error = NULL;
 
 	g_return_if_fail (PK_IS_CONTROL (control));
 	g_return_if_fail (callback != NULL);
@@ -1213,8 +1169,7 @@ pk_control_set_proxy2_async (PkControl *control,
 	if (cancellable != NULL &&
 	    g_cancellable_set_error_if_cancelled (cancellable, &error)) {
 		pk_control_set_proxy_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* skip straight to the D-Bus method if already connection */
@@ -1234,8 +1189,6 @@ pk_control_set_proxy2_async (PkControl *control,
 
 	/* track state */
 	g_ptr_array_add (control->priv->calls, state);
-out:
-	g_object_unref (res);
 }
 
 /**
@@ -1355,11 +1308,11 @@ pk_control_get_transaction_list_cb (GObject *source_object,
 				    GAsyncResult *res,
 				    gpointer user_data)
 {
-	GError *error = NULL;
-	GVariant *value;
 	const gchar **tlist_tmp = NULL;
 	GDBusProxy *proxy = G_DBUS_PROXY (source_object);
 	PkControlState *state = (PkControlState *) user_data;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GVariant) value = NULL;
 
 	/* get the result */
 	value = g_dbus_proxy_call_finish (proxy, res, &error);
@@ -1367,8 +1320,7 @@ pk_control_get_transaction_list_cb (GObject *source_object,
 		/* fix up the D-Bus error */
 		pk_control_fixup_dbus_error (error);
 		pk_control_get_transaction_list_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* unwrap data */
@@ -1382,10 +1334,6 @@ pk_control_get_transaction_list_cb (GObject *source_object,
 
 	/* we're done */
 	pk_control_get_transaction_list_state_finish (state, NULL);
-out:
-	g_free (tlist_tmp);
-	if (value != NULL)
-		g_variant_unref (value);
 }
 
 /**
@@ -1412,13 +1360,12 @@ pk_control_get_transaction_list_proxy_cb (GObject *source_object,
 					  GAsyncResult *res,
 					  gpointer user_data)
 {
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	PkControlState *state = (PkControlState *) user_data;
 
 	state->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 	if (state->proxy == NULL) {
 		pk_control_get_transaction_list_state_finish (state, error);
-		g_error_free (error);
 		return;
 	}
 	pk_control_proxy_connect (state);
@@ -1442,9 +1389,9 @@ pk_control_get_transaction_list_async (PkControl *control,
 				       GAsyncReadyCallback callback,
 				       gpointer user_data)
 {
-	GSimpleAsyncResult *res;
 	PkControlState *state;
-	GError *error = NULL;
+	g_autoptr(GSimpleAsyncResult) res = NULL;
+	g_autoptr(GError) error = NULL;
 
 	g_return_if_fail (PK_IS_CONTROL (control));
 	g_return_if_fail (callback != NULL);
@@ -1466,8 +1413,7 @@ pk_control_get_transaction_list_async (PkControl *control,
 	if (cancellable != NULL &&
 	    g_cancellable_set_error_if_cancelled (cancellable, &error)) {
 		pk_control_get_transaction_list_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* skip straight to the D-Bus method if already connection */
@@ -1487,8 +1433,6 @@ pk_control_get_transaction_list_async (PkControl *control,
 
 	/* track state */
 	g_ptr_array_add (control->priv->calls, state);
-out:
-	g_object_unref (res);
 }
 
 /**
@@ -1571,10 +1515,10 @@ pk_control_get_time_since_action_cb (GObject *source_object,
 				     GAsyncResult *res,
 				     gpointer user_data)
 {
-	GVariant *value;
-	GError *error = NULL;
 	GDBusProxy *proxy = G_DBUS_PROXY (source_object);
 	PkControlState *state = (PkControlState *) user_data;
+	g_autoptr(GVariant) value = NULL;
+	g_autoptr(GError) error = NULL;
 
 	/* get the result */
 	value = g_dbus_proxy_call_finish (proxy, res, &error);
@@ -1582,8 +1526,7 @@ pk_control_get_time_since_action_cb (GObject *source_object,
 		/* fix up the D-Bus error */
 		pk_control_fixup_dbus_error (error);
 		pk_control_get_time_since_action_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* save data */
@@ -1591,15 +1534,11 @@ pk_control_get_time_since_action_cb (GObject *source_object,
 	if (state->time == 0) {
 		error = g_error_new (PK_CONTROL_ERROR, PK_CONTROL_ERROR_FAILED, "could not get time");
 		pk_control_get_time_since_action_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* we're done */
 	pk_control_get_time_since_action_state_finish (state, NULL);
-out:
-	if (value != NULL)
-		g_variant_unref (value);
 }
 
 /**
@@ -1626,13 +1565,12 @@ pk_control_get_time_since_action_proxy_cb (GObject *source_object,
 					   GAsyncResult *res,
 					   gpointer user_data)
 {
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	PkControlState *state = (PkControlState *) user_data;
 
 	state->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 	if (state->proxy == NULL) {
 		pk_control_get_time_since_action_state_finish (state, error);
-		g_error_free (error);
 		return;
 	}
 	pk_control_proxy_connect (state);
@@ -1659,9 +1597,9 @@ pk_control_get_time_since_action_async (PkControl *control,
 					GAsyncReadyCallback callback,
 					gpointer user_data)
 {
-	GSimpleAsyncResult *res;
 	PkControlState *state;
-	GError *error = NULL;
+	g_autoptr(GSimpleAsyncResult) res = NULL;
+	g_autoptr(GError) error = NULL;
 
 	g_return_if_fail (PK_IS_CONTROL (control));
 	g_return_if_fail (callback != NULL);
@@ -1685,8 +1623,7 @@ pk_control_get_time_since_action_async (PkControl *control,
 	if (cancellable != NULL &&
 	    g_cancellable_set_error_if_cancelled (cancellable, &error)) {
 		pk_control_get_time_since_action_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* skip straight to the D-Bus method if already connection */
@@ -1706,8 +1643,6 @@ pk_control_get_time_since_action_async (PkControl *control,
 
 	/* track state */
 	g_ptr_array_add (control->priv->calls, state);
-out:
-	g_object_unref (res);
 }
 
 /**
@@ -1789,10 +1724,10 @@ pk_control_can_authorize_cb (GObject *source_object,
 			     GAsyncResult *res,
 			     gpointer user_data)
 {
-	GError *error = NULL;
 	GDBusProxy *proxy = G_DBUS_PROXY (source_object);
 	PkControlState *state = (PkControlState *) user_data;
-	GVariant *value;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GVariant) value = NULL;
 
 	/* get the result */
 	value = g_dbus_proxy_call_finish (proxy, res, &error);
@@ -1800,8 +1735,7 @@ pk_control_can_authorize_cb (GObject *source_object,
 		/* fix up the D-Bus error */
 		pk_control_fixup_dbus_error (error);
 		pk_control_can_authorize_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* save data */
@@ -1809,15 +1743,11 @@ pk_control_can_authorize_cb (GObject *source_object,
 	if (state->authorize == PK_AUTHORIZE_ENUM_UNKNOWN) {
 		error = g_error_new (PK_CONTROL_ERROR, PK_CONTROL_ERROR_FAILED, "could not get state");
 		pk_control_can_authorize_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* we're done */
 	pk_control_can_authorize_state_finish (state, NULL);
-out:
-	if (value != NULL)
-		g_variant_unref (value);
 }
 
 /**
@@ -1844,13 +1774,12 @@ pk_control_can_authorize_proxy_cb (GObject *source_object,
 				   GAsyncResult *res,
 				   gpointer user_data)
 {
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	PkControlState *state = (PkControlState *) user_data;
 
 	state->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 	if (state->proxy == NULL) {
 		pk_control_can_authorize_state_finish (state, error);
-		g_error_free (error);
 		return;
 	}
 	pk_control_proxy_connect (state);
@@ -1877,9 +1806,9 @@ pk_control_can_authorize_async (PkControl *control,
 				GAsyncReadyCallback callback,
 				gpointer user_data)
 {
-	GSimpleAsyncResult *res;
 	PkControlState *state;
-	GError *error = NULL;
+	g_autoptr(GSimpleAsyncResult) res = NULL;
+	g_autoptr(GError) error = NULL;
 
 	g_return_if_fail (PK_IS_CONTROL (control));
 	g_return_if_fail (callback != NULL);
@@ -1903,8 +1832,7 @@ pk_control_can_authorize_async (PkControl *control,
 	if (cancellable != NULL &&
 	    g_cancellable_set_error_if_cancelled (cancellable, &error)) {
 		pk_control_can_authorize_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 	state->authorize = PK_AUTHORIZE_ENUM_UNKNOWN;
 
@@ -1925,8 +1853,6 @@ pk_control_can_authorize_async (PkControl *control,
 
 	/* track state */
 	g_ptr_array_add (control->priv->calls, state);
-out:
-	g_object_unref (res);
 }
 
 /**
@@ -2005,14 +1931,13 @@ pk_control_get_properties_cb (GObject *source_object,
 			      GAsyncResult *res,
 			      gpointer user_data)
 {
-	GError *error = NULL;
 	PkControlState *state = (PkControlState *) user_data;
+	g_autoptr(GError) error = NULL;
 
 	state->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 	if (state->proxy == NULL) {
 		pk_control_get_properties_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* connect up proxy */
@@ -2023,8 +1948,6 @@ pk_control_get_properties_cb (GObject *source_object,
 
 	/* we're done */
 	pk_control_get_properties_state_finish (state, NULL);
-out:
-	return;
 }
 
 /**
@@ -2044,9 +1967,9 @@ pk_control_get_properties_async (PkControl *control,
 				 GAsyncReadyCallback callback,
 				 gpointer user_data)
 {
-	GSimpleAsyncResult *res;
 	PkControlState *state;
-	GError *error = NULL;
+	g_autoptr(GSimpleAsyncResult) res = NULL;
+	g_autoptr(GError) error = NULL;
 
 	g_return_if_fail (PK_IS_CONTROL (control));
 	g_return_if_fail (callback != NULL);
@@ -2068,15 +1991,14 @@ pk_control_get_properties_async (PkControl *control,
 	if (cancellable != NULL &&
 	    g_cancellable_set_error_if_cancelled (cancellable, &error)) {
 		pk_control_get_properties_state_finish (state, error);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 
 	/* already done */
 	if (control->priv->proxy != NULL) {
 		state->ret = TRUE;
 		pk_control_get_properties_state_finish (state, NULL);
-		goto out;
+		return;
 	}
 
 	/* get a connection to the main interface */
@@ -2092,8 +2014,6 @@ pk_control_get_properties_async (PkControl *control,
 
 	/* track state */
 	g_ptr_array_add (control->priv->calls, state);
-out:
-	g_object_unref (res);
 }
 
 /**
@@ -2178,7 +2098,7 @@ pk_control_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
 		g_value_set_boolean (value, priv->locked);
 		break;
 	case PROP_NETWORK_STATE:
-		g_value_set_uint (value, priv->network_state);
+		g_value_set_enum (value, priv->network_state);
 		break;
 	case PROP_DISTRO_ID:
 		g_value_set_string (value, priv->distro_id);
@@ -2343,8 +2263,8 @@ pk_control_class_init (PkControlClass *klass)
 	 *
 	 * Since: 0.5.3
 	 */
-	pspec = g_param_spec_uint ("network-state", NULL, NULL,
-				   0, G_MAXUINT, PK_NETWORK_ENUM_LAST,
+	pspec = g_param_spec_enum ("network-state", NULL, NULL,
+				   PK_TYPE_NETWORK_ENUM, PK_NETWORK_ENUM_LAST,
 				   G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_NETWORK_STATE, pspec);
 
@@ -2535,6 +2455,7 @@ pk_control_finalize (GObject *object)
 	g_strfreev (priv->mime_types);
 	g_free (priv->distro_id);
 	g_ptr_array_unref (priv->calls);
+	g_object_unref (priv->cancellable);
 
 	G_OBJECT_CLASS (pk_control_parent_class)->finalize (object);
 }
