@@ -31,8 +31,6 @@
 #include "pk-backend.h"
 #include "pk-backend-job.h"
 #include "pk-shared.h"
-#include "pk-sysdep.h"
-#include "pk-time.h"
 
 #define PK_BACKEND_JOB_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PK_TYPE_BACKEND_JOB, PkBackendJobPrivate))
 
@@ -87,7 +85,6 @@ struct PkBackendJobPrivate
 	gchar			*proxy_https;
 	gchar			*proxy_socks;
 	gpointer		 user_data;
-	GThread			*thread;
 	guint64			 download_size_remaining;
 	guint			 cache_age;
 	guint			 download_files;
@@ -96,43 +93,36 @@ struct PkBackendJobPrivate
 	guint			 speed;
 	guint			 uid;
 	GVariant		*params;
+	GCancellable		*cancellable;
 	PkBackend		*backend;
 	PkBackendJobVFuncItem	 vfunc_items[PK_BACKEND_SIGNAL_LAST];
 	PkBitfield		 transaction_flags;
 	GKeyFile		*conf;
 	PkExitEnum		 exit;
-	PkHintEnum		 allow_cancel;
-	PkHintEnum		 background;
-	PkHintEnum		 interactive;
+	gboolean		 allow_cancel;
+	gboolean		 background;
+	gboolean		 interactive;
 	gboolean		 locked;
-	PkPackage		*last_package;
+	GHashTable		*emitted;
 	PkErrorEnum		 last_error_code;
 	PkRoleEnum		 role;
 	PkStatusEnum		 status;
-	PkTime			*time;
+	GTimer			*timer;
 	gboolean		 started;
 };
 
 G_DEFINE_TYPE (PkBackendJob, pk_backend_job, G_TYPE_OBJECT)
 
 /**
- * pk_backend_job_reset:
+ * pk_backend_job_disconnect_vfuncs:
  **/
 void
-pk_backend_job_reset (PkBackendJob *job)
+pk_backend_job_disconnect_vfuncs (PkBackendJob *job)
 {
 	guint i;
 	PkBackendJobVFuncItem *item;
 
-	job->priv->finished = FALSE;
-	job->priv->has_sent_package = FALSE;
-	job->priv->set_error = FALSE;
-	job->priv->thread = NULL;
-	job->priv->exit = PK_EXIT_ENUM_UNKNOWN;
-	job->priv->role = PK_ROLE_ENUM_UNKNOWN;
-	job->priv->status = PK_STATUS_ENUM_UNKNOWN;
-
-	/* reset the vfuncs too */
+	/* reset the vfuncs */
 	for (i = 0; i < PK_BACKEND_SIGNAL_LAST; i++) {
 		item = &job->priv->vfunc_items[i];
 		item->enabled = FALSE;
@@ -157,6 +147,28 @@ pk_backend_job_get_vfunc_enabled (PkBackendJob *job,
 	return TRUE;
 }
 
+/**
+ * pk_backend_job_get_cancellable:
+ *
+ * Return value: (transfer none): a #GCancellable
+ **/
+GCancellable *
+pk_backend_job_get_cancellable (PkBackendJob *job)
+{
+	return job->priv->cancellable;
+}
+
+/**
+ * pk_backend_job_is_cancelled:
+ *
+ * Return value: (transfer none): a #GCancellable
+ **/
+gboolean
+pk_backend_job_is_cancelled (PkBackendJob *job)
+{
+	return g_cancellable_is_cancelled (job->priv->cancellable);
+}
+
 /**
  * pk_backend_job_get_backend:
  *
@@ -245,11 +257,11 @@ pk_backend_job_set_proxy (PkBackendJob	*job,
  *
  * Return value: proxy string in the form username:password@server:port
  **/
-gchar *
+const gchar *
 pk_backend_job_get_proxy_http (PkBackendJob *job)
 {
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), NULL);
-	return g_strdup (job->priv->proxy_http);
+	return job->priv->proxy_http;
 }
 
 /**
@@ -257,11 +269,11 @@ pk_backend_job_get_proxy_http (PkBackendJob *job)
  *
  * Return value: proxy string in the form username:password@server:port
  **/
-gchar *
+const gchar *
 pk_backend_job_get_proxy_https (PkBackendJob *job)
 {
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), NULL);
-	return g_strdup (job->priv->proxy_https);
+	return job->priv->proxy_https;
 }
 
 /**
@@ -269,11 +281,11 @@ pk_backend_job_get_proxy_https (PkBackendJob *job)
  *
  * Return value: proxy string in the form username:password@server:port
  **/
-gchar *
+const gchar *
 pk_backend_job_get_proxy_ftp (PkBackendJob *job)
 {
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), NULL);
-	return g_strdup (job->priv->proxy_ftp);
+	return job->priv->proxy_ftp;
 }
 
 /**
@@ -281,11 +293,11 @@ pk_backend_job_get_proxy_ftp (PkBackendJob *job)
  *
  * Return value: proxy string in the form username:password@server:port
  **/
-gchar *
+const gchar *
 pk_backend_job_get_proxy_socks (PkBackendJob *job)
 {
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), NULL);
-	return g_strdup (job->priv->proxy_socks);
+	return job->priv->proxy_socks;
 }
 
 /**
@@ -293,11 +305,11 @@ pk_backend_job_get_proxy_socks (PkBackendJob *job)
  *
  * Return value: comma seporated value of proxy exlude string
  **/
-gchar *
+const gchar *
 pk_backend_job_get_no_proxy (PkBackendJob *job)
 {
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), NULL);
-	return g_strdup (job->priv->no_proxy);
+	return job->priv->no_proxy;
 }
 
 /**
@@ -305,11 +317,11 @@ pk_backend_job_get_no_proxy (PkBackendJob *job)
  *
  * Return value: proxy PAC filename
  **/
-gchar *
+const gchar *
 pk_backend_job_get_pac (PkBackendJob *job)
 {
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), NULL);
-	return g_strdup (job->priv->pac);
+	return job->priv->pac;
 }
 
 /**
@@ -366,11 +378,11 @@ pk_backend_job_get_uid (PkBackendJob *job)
  *
  * Return value: session locale, e.g. en_GB
  **/
-gchar *
+const gchar *
 pk_backend_job_get_locale (PkBackendJob *job)
 {
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), NULL);
-	return g_strdup (job->priv->locale);
+	return job->priv->locale;
 }
 
 /**
@@ -417,11 +429,11 @@ pk_backend_job_set_parameters (PkBackendJob *job, GVariant *params)
  *
  * Return value: session frontend_socket, e.g. /tmp/socket.345
  **/
-gchar *
+const gchar *
 pk_backend_job_get_frontend_socket (PkBackendJob *job)
 {
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), NULL);
-	return g_strdup (job->priv->frontend_socket);
+	return job->priv->frontend_socket;
 }
 
 /**
@@ -495,10 +507,10 @@ pk_backend_job_set_user_data (PkBackendJob *job, gpointer user_data)
 /**
  * pk_backend_job_get_background:
  **/
-PkHintEnum
+gboolean
 pk_backend_job_get_background (PkBackendJob *job)
 {
-	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), PK_HINT_ENUM_UNSET);
+	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), FALSE);
 	return job->priv->background;
 }
 
@@ -506,7 +518,7 @@ pk_backend_job_get_background (PkBackendJob *job)
  * pk_backend_job_set_background:
  **/
 void
-pk_backend_job_set_background (PkBackendJob *job, PkHintEnum background)
+pk_backend_job_set_background (PkBackendJob *job, gboolean background)
 {
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	job->priv->background = background;
@@ -515,10 +527,10 @@ pk_backend_job_set_background (PkBackendJob *job, PkHintEnum background)
 /**
  * pk_backend_job_get_interactive:
  **/
-PkHintEnum
+gboolean
 pk_backend_job_get_interactive (PkBackendJob *job)
 {
-	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), PK_HINT_ENUM_UNSET);
+	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), FALSE);
 	return job->priv->interactive;
 }
 
@@ -526,7 +538,7 @@ pk_backend_job_get_interactive (PkBackendJob *job)
  * pk_backend_job_set_interactive:
  **/
 void
-pk_backend_job_set_interactive (PkBackendJob *job, PkHintEnum interactive)
+pk_backend_job_set_interactive (PkBackendJob *job, gboolean interactive)
 {
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	job->priv->interactive = interactive;
@@ -551,7 +563,7 @@ guint
 pk_backend_job_get_runtime (PkBackendJob *job)
 {
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), 0);
-	return pk_time_get_elapsed (job->priv->time);
+	return g_timer_elapsed (job->priv->timer, NULL) * 1000;
 }
 
 /**
@@ -604,8 +616,6 @@ pk_backend_job_signal_to_string (PkBackendJobSignal id)
 		return "Files";
 	if (id == PK_BACKEND_SIGNAL_PERCENTAGE)
 		return "Percentage";
-	if (id == PK_BACKEND_SIGNAL_REMAINING)
-		return "Remaining";
 	if (id == PK_BACKEND_SIGNAL_SPEED)
 		return "Speed";
 	if (id == PK_BACKEND_SIGNAL_DOWNLOAD_SIZE_REMAINING)
@@ -631,6 +641,18 @@ pk_backend_job_signal_to_string (PkBackendJobSignal id)
 	return NULL;
 }
 
+/**
+ * pk_backend_job_vfunc_event_free:
+ **/
+static void
+pk_backend_job_vfunc_event_free (PkBackendJobVFuncHelper *helper)
+{
+	if (helper->destroy_func != NULL)
+		helper->destroy_func (helper->object);
+	g_object_unref (helper->job);
+	g_free (helper);
+}
+
 /**
  * pk_backend_job_call_vfunc_idle_cb:
  **/
@@ -643,15 +665,11 @@ pk_backend_job_call_vfunc_idle_cb (gpointer user_data)
 	/* call transaction vfunc on main thread */
 	item = &helper->job->priv->vfunc_items[helper->signal_kind];
 	if (item != NULL && item->vfunc != NULL) {
-		item->vfunc (helper->job,
-			     helper->object,
-			     item->user_data);
+		item->vfunc (helper->job, helper->object, item->user_data);
 	} else {
 		g_warning ("tried to do signal %s when no longer connected",
 			   pk_backend_job_signal_to_string (helper->signal_kind));
 	}
-	if (helper->destroy_func != NULL)
-		helper->destroy_func (helper->object);
 	return FALSE;
 }
 
@@ -670,25 +688,31 @@ pk_backend_job_call_vfunc (PkBackendJob *job,
 	PkBackendJobVFuncHelper *helper;
 	PkBackendJobVFuncItem *item;
 	guint priority = G_PRIORITY_DEFAULT_IDLE;
+	g_autoptr(GSource) source = NULL;
 
 	/* call transaction vfunc if not disabled and set */
 	item = &job->priv->vfunc_items[signal_kind];
 	if (!item->enabled || item->vfunc == NULL)
 		return;
 
+	/* order this last if others are still pending */
 	if (signal_kind == PK_BACKEND_SIGNAL_FINISHED)
 		priority = G_PRIORITY_LOW;
 
-	/* emit idle, TODO: do we ever need to cancel this? */
+	/* emit idle */
 	helper = g_new0 (PkBackendJobVFuncHelper, 1);
-	helper->job = job;
+	helper->job = g_object_ref (job);
 	helper->signal_kind = signal_kind;
 	helper->object = object;
 	helper->destroy_func = destroy_func;
-	g_idle_add_full (priority,
-			 pk_backend_job_call_vfunc_idle_cb,
-			 helper,
-			 g_free);
+	source = g_idle_source_new ();
+	g_source_set_priority (source, priority);
+	g_source_set_callback (source,
+			       pk_backend_job_call_vfunc_idle_cb,
+			       helper,
+			       (GDestroyNotify) pk_backend_job_vfunc_event_free);
+	g_source_set_name (source, "[PkBackendJob] idle_event_cb");
+	g_source_attach (source, NULL);
 }
 
 /**
@@ -730,9 +754,7 @@ pk_backend_job_set_role (PkBackendJob *job, PkRoleEnum role)
 			   pk_role_enum_to_string (job->priv->role));
 	}
 
-	/* reset the timer */
-	pk_time_reset (job->priv->time);
-
+	g_timer_reset (job->priv->timer);
 	job->priv->role = role;
 	job->priv->status = PK_STATUS_ENUM_WAIT;
 	pk_backend_job_call_vfunc (job,
@@ -789,17 +811,16 @@ pk_backend_job_thread_setup (gpointer thread_data)
 	/* run original function with automatic locking */
 	pk_backend_thread_start (helper->backend, helper->job, helper->func);
 	helper->func (helper->job, helper->job->priv->params, helper->user_data);
+	pk_backend_job_finished (helper->job);
 	pk_backend_thread_stop (helper->backend, helper->job, helper->func);
 
 	/* set idle IO priority */
-	if (helper->job->priv->background == PK_HINT_ENUM_TRUE) {
+#ifdef PK_BUILD_DAEMON
+	if (helper->job->priv->background == TRUE) {
 		g_debug ("setting ioprio class to idle");
 		pk_ioprio_set_idle (0);
 	}
-
-	/* unref the thread here as it holds a reference itself and we do
-	 * not need to join() this at any stage */
-	g_thread_unref (helper->job->priv->thread);
+#endif
 
 	/* destroy helper */
 	g_object_unref (helper->job);
@@ -821,16 +842,11 @@ pk_backend_job_thread_create (PkBackendJob *job,
 			      gpointer user_data,
 			      GDestroyNotify destroy_func)
 {
-	gboolean ret = TRUE;
 	PkBackendJobThreadHelper *helper = NULL;
 
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), FALSE);
 	g_return_val_if_fail (func != NULL, FALSE);
-
-	if (job->priv->thread != NULL) {
-		g_warning ("already has thread");
-		return FALSE;
-	}
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
 	/* create a helper object to allow us to call a _setup() function */
 	helper = g_new0 (PkBackendJobThreadHelper, 1);
@@ -839,17 +855,12 @@ pk_backend_job_thread_create (PkBackendJob *job,
 	helper->func = func;
 	helper->user_data = user_data;
 
-	/* create a thread */
-	job->priv->thread = g_thread_new ("PK-Backend",
-					  pk_backend_job_thread_setup,
-					  helper);
-	if (job->priv->thread == NULL) {
-		g_warning ("failed to create thread");
-		ret = FALSE;
-		goto out;
-	}
-out:
-	return ret;
+	/* create a thread and unref it immediately as we do not need to join()
+	 * this at any stage */
+	g_thread_unref (g_thread_new ("PK-Backend",
+	                              pk_backend_job_thread_setup,
+	                              helper));
+	return TRUE;
 }
 
 /**
@@ -858,8 +869,6 @@ out:
 void
 pk_backend_job_set_percentage (PkBackendJob *job, guint percentage)
 {
-	guint remaining;
-
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 
 	/* have we already set an error? */
@@ -893,23 +902,6 @@ pk_backend_job_set_percentage (PkBackendJob *job, guint percentage)
 				   PK_BACKEND_SIGNAL_PERCENTAGE,
 				   GUINT_TO_POINTER (percentage),
 				   NULL);
-
-	/* only compute time if we have data */
-	if (percentage != PK_BACKEND_PERCENTAGE_INVALID) {
-		/* needed for time remaining calculation */
-		pk_time_add_data (job->priv->time, percentage);
-
-		/* lets try this and print as debug */
-		remaining = pk_time_get_remaining (job->priv->time);
-		if (remaining != 0)
-			g_debug ("this will now take ~%i seconds", remaining);
-		job->priv->remaining = remaining;
-		pk_backend_job_call_vfunc (job,
-					   PK_BACKEND_SIGNAL_REMAINING,
-					   GUINT_TO_POINTER (remaining),
-					   NULL);
-	}
-
 }
 
 /**
@@ -1076,9 +1068,10 @@ pk_backend_job_package (PkBackendJob *job,
 			const gchar *package_id,
 			const gchar *summary)
 {
-	PkPackage *item = NULL;
+	PkPackage *emitted_item;
 	gboolean ret;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(PkPackage) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	g_return_if_fail (package_id != NULL);
@@ -1089,27 +1082,25 @@ pk_backend_job_package (PkBackendJob *job,
 	if (!ret) {
 		g_warning ("package_id %s invalid and cannot be processed: %s",
 			   package_id, error->message);
-		g_error_free (error);
-		goto out;
+		return;
 	}
 	pk_package_set_info (item, info);
 	pk_package_set_summary (item, summary);
 
-	/* is it the same? */
-	ret = (job->priv->last_package != NULL &&
-	       pk_package_equal (job->priv->last_package, item));
-	if (ret)
-		goto out;
+	/* already emitted? */
+	emitted_item = g_hash_table_lookup (job->priv->emitted, pk_package_get_id (item));
+	if (emitted_item != NULL && pk_package_equal (emitted_item, item))
+		return;
 
-	/* update the 'last' package */
-	if (job->priv->last_package != NULL)
-		g_object_unref (job->priv->last_package);
-	job->priv->last_package = g_object_ref (item);
+	/* update the emitted package table */
+	g_hash_table_insert (job->priv->emitted,
+	                     g_strdup (pk_package_get_id (item)),
+	                     g_object_ref (item));
 
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: package %s", package_id);
-		goto out;
+		return;
 	}
 
 	/* we automatically set the transaction status  */
@@ -1134,9 +1125,6 @@ pk_backend_job_package (PkBackendJob *job,
 				   PK_BACKEND_SIGNAL_PACKAGE,
 				   g_object_ref (item),
 				   g_object_unref);
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1157,9 +1145,8 @@ pk_backend_job_update_detail (PkBackendJob *job,
 			      const gchar *issued_text,
 			      const gchar *updated_text)
 {
-	PkUpdateDetail *item = NULL;
 	GTimeVal timeval;
-	gboolean ret;
+	g_autoptr(PkUpdateDetail) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	g_return_if_fail (package_id != NULL);
@@ -1167,7 +1154,7 @@ pk_backend_job_update_detail (PkBackendJob *job,
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: update_detail %s", package_id);
-		goto out;
+		return;
 	}
 
 	/* check the dates are not empty */
@@ -1178,13 +1165,11 @@ pk_backend_job_update_detail (PkBackendJob *job,
 
 	/* check the issued dates are valid */
 	if (issued_text != NULL) {
-		ret = g_time_val_from_iso8601 (issued_text, &timeval);
-		if (!ret)
+		if (!g_time_val_from_iso8601 (issued_text, &timeval))
 			g_warning ("failed to parse issued '%s'", issued_text);
 	}
 	if (updated_text != NULL) {
-		ret = g_time_val_from_iso8601 (updated_text, &timeval);
-		if (!ret)
+		if (!g_time_val_from_iso8601 (updated_text, &timeval))
 			g_warning ("failed to parse updated '%s'", updated_text);
 	}
 
@@ -1210,9 +1195,6 @@ pk_backend_job_update_detail (PkBackendJob *job,
 				   PK_BACKEND_SIGNAL_UPDATE_DETAIL,
 				   g_object_ref (item),
 				   g_object_unref);
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1223,22 +1205,20 @@ pk_backend_job_require_restart (PkBackendJob *job,
 				PkRestartEnum restart,
 				const gchar *package_id)
 {
-	gboolean ret;
-	PkRequireRestart *item = NULL;
+	g_autoptr(PkRequireRestart) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: require-restart %s", pk_restart_enum_to_string (restart));
-		goto out;
+		return;
 	}
 
 	/* check we are valid */
-	ret = pk_package_id_check (package_id);
-	if (!ret) {
+	if (!pk_package_id_check (package_id)) {
 		g_warning ("package_id invalid and cannot be processed: %s", package_id);
-		goto out;
+		return;
 	}
 
 	/* form PkRequireRestart struct */
@@ -1253,9 +1233,6 @@ pk_backend_job_require_restart (PkBackendJob *job,
 				   PK_BACKEND_SIGNAL_REQUIRE_RESTART,
 				   g_object_ref (item),
 				   g_object_unref);
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1271,7 +1248,7 @@ pk_backend_job_details (PkBackendJob *job,
 			const gchar *url,
 			gulong size)
 {
-	PkDetails *item = NULL;
+	g_autoptr(PkDetails) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	g_return_if_fail (package_id != NULL);
@@ -1279,7 +1256,7 @@ pk_backend_job_details (PkBackendJob *job,
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: details %s", package_id);
-		goto out;
+		return;
 	}
 
 	/* form PkDetails struct */
@@ -1299,9 +1276,6 @@ pk_backend_job_details (PkBackendJob *job,
 			       PK_BACKEND_SIGNAL_DETAILS,
 			       g_object_ref (item),
 				   g_object_unref);
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1314,8 +1288,7 @@ pk_backend_job_files (PkBackendJob *job,
 		      const gchar *package_id,
 		      gchar **files)
 {
-	gboolean ret;
-	PkFiles *item = NULL;
+	g_autoptr(PkFiles) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	g_return_if_fail (files != NULL);
@@ -1323,15 +1296,14 @@ pk_backend_job_files (PkBackendJob *job,
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: files %s", package_id);
-		goto out;
+		return;
 	}
 
 	/* check we are valid if specified */
 	if (package_id != NULL) {
-		ret = pk_package_id_check (package_id);
-		if (!ret) {
+		if (!pk_package_id_check (package_id)) {
 			g_warning ("package_id invalid and cannot be processed: %s", package_id);
-			goto out;
+			return;
 		}
 	}
 
@@ -1350,9 +1322,6 @@ pk_backend_job_files (PkBackendJob *job,
 
 	/* success */
 	job->priv->download_files++;
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1364,7 +1333,7 @@ pk_backend_job_distro_upgrade (PkBackendJob *job,
 			       const gchar *name,
 			       const gchar *summary)
 {
-	PkDistroUpgrade *item = NULL;
+	g_autoptr(PkDistroUpgrade) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	g_return_if_fail (state != PK_DISTRO_UPGRADE_ENUM_UNKNOWN);
@@ -1374,7 +1343,7 @@ pk_backend_job_distro_upgrade (PkBackendJob *job,
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: distro-upgrade");
-		goto out;
+		return;
 	}
 
 	/* form PkDistroUpgrade struct */
@@ -1390,9 +1359,6 @@ pk_backend_job_distro_upgrade (PkBackendJob *job,
 				   PK_BACKEND_SIGNAL_DISTRO_UPGRADE,
 				   g_object_ref (item),
 				   g_object_unref);
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1409,7 +1375,7 @@ pk_backend_job_repo_signature_required (PkBackendJob *job,
 					const gchar *key_timestamp,
 					PkSigTypeEnum type)
 {
-	PkRepoSignatureRequired *item = NULL;
+	g_autoptr(PkRepoSignatureRequired) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	g_return_if_fail (repository_name != NULL);
@@ -1417,13 +1383,13 @@ pk_backend_job_repo_signature_required (PkBackendJob *job,
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: repo-sig-reqd");
-		goto out;
+		return;
 	}
 
 	/* check we don't do this more than once */
 	if (job->priv->set_signature) {
 		g_warning ("already asked for a signature, cannot process");
-		goto out;
+		return;
 	}
 
 	/* form PkRepoSignatureRequired struct */
@@ -1447,9 +1413,6 @@ pk_backend_job_repo_signature_required (PkBackendJob *job,
 
 	/* success */
 	job->priv->set_signature = TRUE;
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1462,7 +1425,7 @@ pk_backend_job_eula_required (PkBackendJob *job,
 			      const gchar *vendor_name,
 			      const gchar *license_agreement)
 {
-	PkEulaRequired *item = NULL;
+	g_autoptr(PkEulaRequired) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	g_return_if_fail (eula_id != NULL);
@@ -1473,13 +1436,13 @@ pk_backend_job_eula_required (PkBackendJob *job,
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: eula required");
-		goto out;
+		return;
 	}
 
 	/* check we don't do this more than once */
 	if (job->priv->set_eula) {
 		g_warning ("already asked for a signature, cannot process");
-		goto out;
+		return;
 	}
 
 	/* form PkEulaRequired struct */
@@ -1499,9 +1462,6 @@ pk_backend_job_eula_required (PkBackendJob *job,
 
 	/* success */
 	job->priv->set_eula = TRUE;
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1513,7 +1473,7 @@ pk_backend_job_media_change_required (PkBackendJob *job,
 				      const gchar *media_id,
 				      const gchar *media_text)
 {
-	PkMediaChangeRequired *item = NULL;
+	g_autoptr(PkMediaChangeRequired) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	g_return_if_fail (media_id != NULL);
@@ -1522,7 +1482,7 @@ pk_backend_job_media_change_required (PkBackendJob *job,
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: media change required");
-		goto out;
+		return;
 	}
 
 	/* form PkMediaChangeRequired struct */
@@ -1538,9 +1498,6 @@ pk_backend_job_media_change_required (PkBackendJob *job,
 				   PK_BACKEND_SIGNAL_MEDIA_CHANGE_REQUIRED,
 				   g_object_ref (item),
 				   g_object_unref);
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1552,7 +1509,7 @@ pk_backend_job_repo_detail (PkBackendJob *job,
 			    const gchar *description,
 			    gboolean enabled)
 {
-	PkRepoDetail *item = NULL;
+	g_autoptr(PkRepoDetail) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	g_return_if_fail (repo_id != NULL);
@@ -1560,7 +1517,7 @@ pk_backend_job_repo_detail (PkBackendJob *job,
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: repo-detail %s", repo_id);
-		goto out;
+		return;
 	}
 
 	/* form PkRepoDetail struct */
@@ -1576,9 +1533,6 @@ pk_backend_job_repo_detail (PkBackendJob *job,
 				   PK_BACKEND_SIGNAL_REPO_DETAIL,
 				   g_object_ref (item),
 				   g_object_unref);
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1592,7 +1546,7 @@ pk_backend_job_category (PkBackendJob *job,
 			 const gchar *summary,
 			 const gchar *icon)
 {
-	PkCategory *item = NULL;
+	g_autoptr(PkCategory) item = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 	g_return_if_fail (cat_id != NULL);
@@ -1600,7 +1554,7 @@ pk_backend_job_category (PkBackendJob *job,
 	/* have we already set an error? */
 	if (job->priv->set_error) {
 		g_warning ("already set error: category %s", cat_id);
-		goto out;
+		return;
 	}
 
 	/* form PkCategory struct */
@@ -1618,9 +1572,6 @@ pk_backend_job_category (PkBackendJob *job,
 				   PK_BACKEND_SIGNAL_CATEGORY,
 				   g_object_ref (item),
 				   g_object_unref);
-out:
-	if (item != NULL)
-		g_object_unref (item);
 }
 
 /**
@@ -1653,9 +1604,9 @@ pk_backend_job_error_code (PkBackendJob *job,
 			   const gchar *format, ...)
 {
 	va_list args;
-	gchar *buffer;
 	gboolean need_untrusted;
-	PkError *error = NULL;
+	g_autofree gchar *buffer = NULL;
+	g_autoptr(PkError) error = NULL;
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 
@@ -1672,7 +1623,7 @@ pk_backend_job_error_code (PkBackendJob *job,
 			job->priv->finished = FALSE;
 		} else {
 			g_warning ("More than one error emitted! You tried to set '%s'", buffer);
-			goto out;
+			return;
 		}
 	}
 	job->priv->set_error = TRUE;
@@ -1683,7 +1634,7 @@ pk_backend_job_error_code (PkBackendJob *job,
 		pk_backend_job_set_exit_code (job, PK_EXIT_ENUM_NEED_UNTRUSTED);
 	else if (error_code == PK_ERROR_ENUM_CANCELLED_PRIORITY)
 		pk_backend_job_set_exit_code (job, PK_EXIT_ENUM_CANCELLED_PRIORITY);
-	else
+	else if (job->priv->exit == PK_EXIT_ENUM_UNKNOWN)
 		pk_backend_job_set_exit_code (job, PK_EXIT_ENUM_FAILED);
 
 	/* set the hint that RepairSystem is needed */
@@ -1706,10 +1657,6 @@ pk_backend_job_error_code (PkBackendJob *job,
 				   PK_BACKEND_SIGNAL_ERROR_CODE,
 				   g_object_ref (error),
 				   g_object_unref);
-out:
-	if (error != NULL)
-		g_object_unref (error);
-	g_free (buffer);
 }
 
 /**
@@ -1748,22 +1695,16 @@ pk_backend_job_set_allow_cancel (PkBackendJob *job, gboolean allow_cancel)
 {
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 
-	/* have we already set an error? */
-	if (job->priv->set_error && allow_cancel) {
-		g_warning ("already set error: allow-cancel %i", allow_cancel);
-		return;
-	}
-
 	/* same as last state? */
-	if (job->priv->allow_cancel == (PkHintEnum) allow_cancel)
+	if (job->priv->allow_cancel == allow_cancel)
 		return;
 
 	/* emit */
+	job->priv->allow_cancel = allow_cancel;
 	pk_backend_job_call_vfunc (job,
 				   PK_BACKEND_SIGNAL_ALLOW_CANCEL,
 				   GUINT_TO_POINTER (allow_cancel),
 				   NULL);
-	job->priv->allow_cancel = allow_cancel;
 }
 
 /**
@@ -1772,33 +1713,8 @@ pk_backend_job_set_allow_cancel (PkBackendJob *job, gboolean allow_cancel)
 gboolean
 pk_backend_job_get_allow_cancel (PkBackendJob *job)
 {
-	gboolean allow_cancel = FALSE;
-
 	g_return_val_if_fail (PK_IS_BACKEND_JOB (job), FALSE);
-
-	/* return FALSE if we never set state */
-	if (job->priv->allow_cancel != PK_HINT_ENUM_UNSET)
-		allow_cancel = job->priv->allow_cancel;
-
-	return allow_cancel;
-}
-
-/**
- * pk_backend_job_not_implemented_yet:
- **/
-void
-pk_backend_job_not_implemented_yet (PkBackendJob *job, const gchar *method)
-{
-	g_return_if_fail (PK_IS_BACKEND_JOB (job));
-	g_return_if_fail (method != NULL);
-
-	pk_backend_job_error_code (job,
-				   PK_ERROR_ENUM_NOT_SUPPORTED,
-				   "the method '%s' is not implemented yet",
-				   method);
-
-	/* don't wait, do this now */
-	pk_backend_job_finished (job);
+	return job->priv->allow_cancel;
 }
 
 /**
@@ -1832,21 +1748,6 @@ pk_backend_job_get_exit_code (PkBackendJob *job)
 	return job->priv->exit;
 }
 
-/**
- * pk_backend_job_use_background:
- **/
-gboolean
-pk_backend_job_use_background (PkBackendJob *job)
-{
-	/* the session has set it one way or the other */
-	if (job->priv->background == PK_HINT_ENUM_TRUE)
-		return TRUE;
-	if (job->priv->background == PK_HINT_ENUM_FALSE)
-		return FALSE;
-
-	return FALSE;
-}
-
 /**
  * pk_backend_job_finished:
  **/
@@ -1857,15 +1758,15 @@ pk_backend_job_finished (PkBackendJob *job)
 
 	g_return_if_fail (PK_IS_BACKEND_JOB (job));
 
-	/* find out what we just did */
-	role_text = pk_role_enum_to_string (job->priv->role);
-
 	/* check we have not already finished */
 	if (job->priv->finished) {
 		g_warning ("already finished");
 		return;
 	}
 
+	/* find out what we just did */
+	role_text = pk_role_enum_to_string (job->priv->role);
+
 	/* ensure the same number of ::Files() were sent as packages for DownloadPackages */
 	if (!job->priv->set_error &&
 	    job->priv->role == PK_ROLE_ENUM_DOWNLOAD_PACKAGES &&
@@ -1879,8 +1780,8 @@ pk_backend_job_finished (PkBackendJob *job)
 		g_warning ("required status signals for %s!", role_text);
 	}
 
-	/* make any UI insensitive */
-	pk_backend_job_set_allow_cancel (job, FALSE);
+	/* drop any inhibits */
+	pk_backend_job_set_allow_cancel (job, TRUE);
 
 	/* mark as finished for the UI that might only be watching status */
 	pk_backend_job_set_status (job, PK_STATUS_ENUM_FINISHED);
@@ -1909,6 +1810,7 @@ pk_backend_job_finalize (GObject *object)
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (PK_IS_BACKEND_JOB (object));
+	g_return_if_fail (pk_is_thread_default ());
 	job = PK_BACKEND_JOB (object);
 
 	if (pk_backend_job_get_started (job)) {
@@ -1925,14 +1827,12 @@ pk_backend_job_finalize (GObject *object)
 	g_free (job->priv->cmdline);
 	g_free (job->priv->locale);
 	g_free (job->priv->frontend_socket);
-	if (job->priv->last_package != NULL) {
-		g_object_unref (job->priv->last_package);
-		job->priv->last_package = NULL;
-	}
+	g_hash_table_unref (job->priv->emitted);
 	if (job->priv->params != NULL)
 		g_variant_unref (job->priv->params);
-	g_object_unref (job->priv->time);
+	g_timer_destroy (job->priv->timer);
 	g_key_file_unref (job->priv->conf);
+	g_object_unref (job->priv->cancellable);
 
 	G_OBJECT_CLASS (pk_backend_job_parent_class)->finalize (object);
 }
@@ -1955,9 +1855,17 @@ static void
 pk_backend_job_init (PkBackendJob *job)
 {
 	job->priv = PK_BACKEND_JOB_GET_PRIVATE (job);
-	job->priv->time = pk_time_new ();
+	job->priv->timer = g_timer_new ();
+	job->priv->cancellable = g_cancellable_new ();
 	job->priv->last_error_code = PK_ERROR_ENUM_UNKNOWN;
-	pk_backend_job_reset (job);
+	job->priv->locale = g_strdup ("C");
+	job->priv->cache_age = G_MAXUINT;
+	job->priv->allow_cancel = TRUE;
+	job->priv->exit = PK_EXIT_ENUM_UNKNOWN;
+	job->priv->role = PK_ROLE_ENUM_UNKNOWN;
+	job->priv->status = PK_STATUS_ENUM_UNKNOWN;
+	job->priv->emitted = g_hash_table_new_full (g_str_hash, g_str_equal,
+	                                            g_free, (GDestroyNotify) g_object_unref);
 }
 
 /**
