@@ -30,14 +30,13 @@
 #include <glib/gi18n.h>
 #include <glib.h>
 #include <gmodule.h>
+#include <packagekit-glib2/pk-offline-private.h>
 #include <packagekit-glib2/pk-package-id.h>
 #include <packagekit-glib2/pk-results.h>
 #include <packagekit-glib2/pk-common.h>
 
-#include "pk-network.h"
 #include "pk-backend.h"
 #include "pk-shared.h"
-#include "pk-notify.h"
 
 #define PK_BACKEND_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PK_TYPE_BACKEND, PkBackendPrivate))
 
@@ -66,8 +65,6 @@ typedef struct {
 	gboolean	(*supports_parallelization)	(PkBackend	*backend);
 	void		(*job_start)			(PkBackend	*backend,
 							 PkBackendJob	*job);
-	void		(*job_reset)			(PkBackend	*backend,
-							 PkBackendJob	*job);
 	void		(*job_stop)			(PkBackend	*backend,
 							 PkBackendJob	*job);
 	void		(*cancel)			(PkBackend	*backend,
@@ -178,6 +175,11 @@ typedef struct {
 							 PkBackendJob	*job,
 							 PkBitfield	 filters,
 							 gchar		**values);
+	void		(*upgrade_system)		(PkBackend	*backend,
+							 PkBackendJob	*job,
+							 PkBitfield	 transaction_flags,
+							 const gchar	*distro_id,
+							 PkUpgradeKindEnum upgrade_kind);
 	void		(*repair_system)		(PkBackend	*backend,
 							 PkBackendJob	*job,
 							 PkBitfield	 transaction_flags);
@@ -196,14 +198,27 @@ struct PkBackendPrivate
 	PkBitfield		 roles;
 	GKeyFile		*conf;
 	GFileMonitor		*monitor;
-	PkNetwork		*network;
 	gboolean		 backend_roles_set;
+	gpointer		 user_data;
 	GHashTable		*thread_hash;
 	GMutex			 thread_hash_mutex;
+	gboolean		 transaction_in_progress;
+	guint			 transaction_inhibit_end_idle_id;
+	guint			 repo_list_changed_id;
+	guint			 installed_db_changed_id;
+	guint			 updates_changed_id;
 };
 
 G_DEFINE_TYPE (PkBackend, pk_backend, G_TYPE_OBJECT)
 
+enum {
+	SIGNAL_REPO_LIST_CHANGED,
+	SIGNAL_UPDATES_CHANGED,
+	SIGNAL_LAST
+};
+
+static guint signals [SIGNAL_LAST] = { 0 };
+
 /**
  * pk_backend_get_groups:
  **/
@@ -212,6 +227,7 @@ pk_backend_get_groups (PkBackend *backend)
 {
 	g_return_val_if_fail (PK_IS_BACKEND (backend), PK_GROUP_ENUM_UNKNOWN);
 	g_return_val_if_fail (backend->priv->loaded, PK_GROUP_ENUM_UNKNOWN);
+	g_return_val_if_fail (pk_is_thread_default (), PK_GROUP_ENUM_UNKNOWN);
 
 	/* not compulsory */
 	if (backend->priv->desc->get_groups == NULL)
@@ -229,6 +245,7 @@ pk_backend_get_mime_types (PkBackend *backend)
 {
 	g_return_val_if_fail (PK_IS_BACKEND (backend), NULL);
 	g_return_val_if_fail (backend->priv->loaded, NULL);
+	g_return_val_if_fail (pk_is_thread_default (), NULL);
 
 	/* not compulsory */
 	if (backend->priv->desc->get_mime_types == NULL)
@@ -296,6 +313,7 @@ pk_backend_get_filters (PkBackend *backend)
 {
 	g_return_val_if_fail (PK_IS_BACKEND (backend), PK_FILTER_ENUM_UNKNOWN);
 	g_return_val_if_fail (backend->priv->loaded, PK_FILTER_ENUM_UNKNOWN);
+	g_return_val_if_fail (pk_is_thread_default (), PK_FILTER_ENUM_UNKNOWN);
 
 	/* not compulsory */
 	if (backend->priv->desc->get_filters == NULL)
@@ -314,6 +332,7 @@ pk_backend_get_roles (PkBackend *backend)
 
 	g_return_val_if_fail (PK_IS_BACKEND (backend), PK_ROLE_ENUM_UNKNOWN);
 	g_return_val_if_fail (backend->priv->loaded, PK_ROLE_ENUM_UNKNOWN);
+	g_return_val_if_fail (pk_is_thread_default (), PK_ROLE_ENUM_UNKNOWN);
 
 	/* optimise - we only skip here if we already loaded backend settings,
 	 * so we don't override preexisting settings (e.g. by plugins) */
@@ -387,6 +406,8 @@ pk_backend_get_roles (PkBackend *backend)
 		pk_bitfield_add (roles, PK_ROLE_ENUM_GET_DISTRO_UPGRADES);
 	if (desc->get_categories != NULL)
 		pk_bitfield_add (roles, PK_ROLE_ENUM_GET_CATEGORIES);
+	if (desc->upgrade_system != NULL)
+		pk_bitfield_add (roles, PK_ROLE_ENUM_UPGRADE_SYSTEM);
 	if (desc->repair_system != NULL)
 		pk_bitfield_add (roles, PK_ROLE_ENUM_REPAIR_SYSTEM);
 	pk_bitfield_add (roles, PK_ROLE_ENUM_GET_OLD_TRANSACTIONS);
@@ -409,17 +430,6 @@ pk_backend_is_implemented (PkBackend *backend, PkRoleEnum role)
 	return pk_bitfield_contain (roles, role);
 }
 
-/**
- * pk_backend_implement:
- **/
-void
-pk_backend_implement (PkBackend *backend, PkRoleEnum role)
-{
-	g_return_if_fail (PK_IS_BACKEND (backend));
-	g_return_if_fail (role != PK_ROLE_ENUM_UNKNOWN);
-	pk_bitfield_add (backend->priv->roles, role);
-}
-
 /**
  * pk_backend_build_library_path:
  **/
@@ -427,7 +437,7 @@ static gchar *
 pk_backend_build_library_path (PkBackend *backend, const gchar *name)
 {
 	gchar *path;
-	gchar *filename;
+	g_autofree gchar *filename = NULL;
 #if PK_BUILD_LOCAL
 	const gchar *directory;
 #endif
@@ -451,7 +461,6 @@ pk_backend_build_library_path (PkBackend *backend, const gchar *name)
 #else
 	path = g_build_filename (LIBDIR, "packagekit-backend", filename, NULL);
 #endif
-	g_free (filename);
 	g_debug ("dlopening '%s'", path);
 
 	return path;
@@ -473,19 +482,20 @@ gboolean
 pk_backend_load (PkBackend *backend, GError **error)
 {
 	GModule *handle;
-	gchar *path = NULL;
 	gboolean ret = FALSE;
 	gpointer func = NULL;
-	gchar *backend_name = NULL;
+	g_autofree gchar *backend_name = NULL;
+	g_autofree gchar *path = NULL;
 
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
 	/* already loaded */
 	if (backend->priv->loaded) {
 		g_set_error (error, 1, 0,
 			     "already set name to %s",
 			     backend->priv->name);
-		goto out;
+		return FALSE;
 	}
 
 	/* can we load it? */
@@ -494,14 +504,21 @@ pk_backend_load (PkBackend *backend, GError **error)
 					      "DefaultBackend",
 					      error);
 	if (backend_name == NULL)
-		goto out;
+		return FALSE;
+
+	/* the "hawkey" backend was renamed to "hif" */
+	if (g_strcmp0 (backend_name, "hawkey") == 0) {
+		g_free (backend_name);
+		backend_name = g_strdup ("hif");
+	}
+
 	g_debug ("Trying to load : %s", backend_name);
 	path = pk_backend_build_library_path (backend, backend_name);
 	handle = g_module_open (path, 0);
 	if (handle == NULL) {
 		g_set_error (error, 1, 0, "opening module %s failed : %s",
 			     backend_name, g_module_error ());
-		goto out;
+		return FALSE;
 	}
 
 	/* then check for the new style exported functions */
@@ -549,9 +566,9 @@ pk_backend_load (PkBackend *backend, GError **error)
 		g_module_symbol (handle, "pk_backend_search_names", (gpointer *)&desc->search_names);
 		g_module_symbol (handle, "pk_backend_start_job", (gpointer *)&desc->job_start);
 		g_module_symbol (handle, "pk_backend_stop_job", (gpointer *)&desc->job_stop);
-		g_module_symbol (handle, "pk_backend_reset_job", (gpointer *)&desc->job_reset);
 		g_module_symbol (handle, "pk_backend_update_packages", (gpointer *)&desc->update_packages);
 		g_module_symbol (handle, "pk_backend_what_provides", (gpointer *)&desc->what_provides);
+		g_module_symbol (handle, "pk_backend_upgrade_system", (gpointer *)&desc->upgrade_system);
 		g_module_symbol (handle, "pk_backend_repair_system", (gpointer *)&desc->repair_system);
 
 		/* get old static string data */
@@ -566,8 +583,10 @@ pk_backend_load (PkBackend *backend, GError **error)
 		backend->priv->desc = desc;
 	} else {
 		g_module_close (handle);
-		g_set_error (error, 1, 0, "could not find description in plugin %s, not loading", backend_name);
-		goto out;
+		g_set_error (error, 1, 0,
+			     "could not find description in plugin %s, not loading",
+			     backend_name);
+		return FALSE;
 	}
 
 	/* save the backend name and handle */
@@ -582,10 +601,7 @@ pk_backend_load (PkBackend *backend, GError **error)
 		backend->priv->during_initialize = FALSE;
 	}
 	backend->priv->loaded = TRUE;
-out:
-	g_free (path);
-	g_free (backend_name);
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -601,6 +617,7 @@ gboolean
 pk_backend_unload (PkBackend *backend)
 {
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
 	if (backend->priv->loaded == FALSE) {
 		g_debug ("already closed (nonfatal)");
@@ -618,23 +635,188 @@ pk_backend_unload (PkBackend *backend)
 }
 
 /**
- * pk_backend_repo_list_changed:
+ * pk_backend_repo_list_changed_cb:
  **/
-gboolean
+static gboolean
+pk_backend_repo_list_changed_cb (gpointer user_data)
+{
+	PkBackend *backend = PK_BACKEND (user_data);
+
+	g_debug ("emitting repo-list-changed");
+	g_signal_emit (backend, signals [SIGNAL_REPO_LIST_CHANGED], 0);
+	backend->priv->repo_list_changed_id = 0;
+	return FALSE;
+}
+
+/**
+ * pk_backend_repo_list_changed:
+ *
+ * This function can be called on any thread.
+ **/
+void
 pk_backend_repo_list_changed (PkBackend *backend)
 {
-	PkNotify *notify;
+	g_return_if_fail (PK_IS_BACKEND (backend));
+	g_return_if_fail (backend->priv->loaded);
 
+	/* already scheduled */
+	if (backend->priv->repo_list_changed_id != 0)
+		return;
+
+	/* idle add */
+	backend->priv->repo_list_changed_id =
+		g_idle_add (pk_backend_repo_list_changed_cb, backend);
+
+}
+/**
+ * pk_backend_updates_changed:
+ **/
+gboolean
+pk_backend_updates_changed (PkBackend *backend)
+{
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
-	g_return_val_if_fail (backend->priv->loaded, FALSE);
-
-	notify = pk_notify_new ();
-	pk_notify_repo_list_changed (notify);
-	g_object_unref (notify);
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
+	g_debug ("emitting updates-changed");
+	g_signal_emit (backend, signals [SIGNAL_UPDATES_CHANGED], 0);
 	return TRUE;
 }
 
+/**
+ * pk_backend_finished_updates_changed_cb:
+ **/
+static gboolean
+pk_backend_finished_updates_changed_cb (gpointer data)
+{
+	PkBackend *backend = PK_BACKEND (data);
+	pk_backend_updates_changed (backend);
+	backend->priv->updates_changed_id = 0;
+	return FALSE;
+}
+
+/**
+ * pk_backend_updates_changed_delay:
+ *
+ * This function can be called on any thread.
+ **/
+gboolean
+pk_backend_updates_changed_delay (PkBackend *backend, guint timeout)
+{
+	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
+
+	/* check if we did this more than once */
+	if (backend->priv->updates_changed_id != 0)
+		return FALSE;
+
+	/* schedule */
+	backend->priv->updates_changed_id =
+		g_timeout_add (timeout,
+			       pk_backend_finished_updates_changed_cb,
+			       backend);
+	g_source_set_name_by_id (backend->priv->updates_changed_id,
+				 "[PkBackend] updates-changed");
+	return TRUE;
+}
+
+/**
+ * pk_backend_installed_db_changed_cb:
+ **/
+static gboolean
+pk_backend_installed_db_changed_cb (gpointer user_data)
+{
+	PkBackend *backend = PK_BACKEND (user_data);
+	g_autoptr(GError) error = NULL;
+
+	if (!backend->priv->transaction_in_progress) {
+		g_debug ("invalidating offline updates");
+		if (!pk_offline_auth_invalidate (&error))
+			g_warning ("failed to invalidate: %s", error->message);
+	}
+	backend->priv->installed_db_changed_id = 0;
+	return FALSE;
+}
+
+/**
+ * pk_backend_installed_db_changed:
+ *
+ * This method signals PackageKit to drop any internal caches that should be
+ * invalidated when something external changes the package database, e.g. a
+ * native command line package management tool.
+ *
+ * Typically, a backend would set up a file monitor and call this method when
+ * the package database file has changed. To avoid invalidating caches for
+ * transactions done by PackageKit itself, a backend would call
+ * pk_backend_transaction_inhibit_start() before each transaction and
+ * pk_backend_transaction_inhibit_end() after the transaction has finished.
+ *
+ * This function can be called on any thread.
+ **/
+void
+pk_backend_installed_db_changed (PkBackend *backend)
+{
+	g_return_if_fail (PK_IS_BACKEND (backend));
+	g_return_if_fail (backend->priv->loaded);
+
+	/* already scheduled */
+	if (backend->priv->installed_db_changed_id != 0)
+		return;
+
+	/* idle add */
+	backend->priv->installed_db_changed_id =
+		g_idle_add (pk_backend_installed_db_changed_cb, backend);
+}
+
+/**
+ * pk_backend_transaction_inhibit_start:
+ *
+ * Call this method before each transaction that touches the installed package
+ * database.
+ **/
+void
+pk_backend_transaction_inhibit_start (PkBackend *backend)
+{
+	backend->priv->transaction_in_progress = TRUE;
+}
+
+static gboolean
+transaction_inhibit_end_idle (gpointer user_data)
+{
+	PkBackend *backend = user_data;
+
+	backend->priv->transaction_in_progress = FALSE;
+	backend->priv->transaction_inhibit_end_idle_id = 0;
+
+	return G_SOURCE_REMOVE;
+}
+
+/**
+ * pk_backend_transaction_inhibit_end:
+ *
+ * Call this method after each transaction that touches the installed package
+ * database.
+ **/
+void
+pk_backend_transaction_inhibit_end (PkBackend *backend)
+{
+	if (backend->priv->transaction_inhibit_end_idle_id > 0)
+		g_source_remove (backend->priv->transaction_inhibit_end_idle_id);
+
+	/* delay for 3 seconds in order to cover the 2 second rate limit
+           timeout used by the gio file monitor */
+	backend->priv->transaction_inhibit_end_idle_id = g_timeout_add_seconds (3,
+	                                                                        transaction_inhibit_end_idle,
+	                                                                        backend);
+}
+
+/**
+ * pk_backend_is_transaction_inhibited:
+ **/
+gboolean
+pk_backend_is_transaction_inhibited (PkBackend *backend)
+{
+	return backend->priv->transaction_in_progress;
+}
+
 /**
  * pk_backend_start_job:
  *
@@ -672,6 +854,7 @@ void
 pk_backend_start_job (PkBackend *backend, PkBackendJob *job)
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* common stuff */
 	pk_backend_job_set_backend (job, backend);
@@ -688,33 +871,6 @@ pk_backend_start_job (PkBackend *backend, PkBackendJob *job)
 		backend->priv->desc->job_start (backend, job);
 }
 
-/**
- * pk_backend_reset_job:
- **/
-void
-pk_backend_reset_job (PkBackend *backend, PkBackendJob *job)
-{
-	g_return_if_fail (PK_IS_BACKEND (backend));
-
-	if (!pk_backend_job_get_started (job)) {
-		g_warning ("trying to reset job, but never started it before");
-		return;
-	}
-
-	/* optional */
-	if (backend->priv->desc->job_reset != NULL) {
-		backend->priv->desc->job_reset (backend, job);
-	} else {
-		if (backend->priv->desc->job_stop != NULL)
-			backend->priv->desc->job_stop (backend, job);
-		if (backend->priv->desc->job_start != NULL)
-			backend->priv->desc->job_start (backend, job);
-	}
-
-	/* bubble up */
-	pk_backend_job_reset (job);
-}
-
 /**
  * pk_backend_stop_job:
  *
@@ -728,6 +884,7 @@ void
 pk_backend_stop_job (PkBackend *backend, PkBackendJob *job)
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
+	g_return_if_fail (pk_is_thread_default ());
 
 	if (!pk_backend_job_get_started (job)) {
 		g_warning ("trying to stop job, but never started it before");
@@ -758,15 +915,35 @@ pk_backend_bool_to_string (gboolean value)
 gboolean
 pk_backend_is_online (PkBackend *backend)
 {
-	PkNetworkEnum state;
-	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
-	state = pk_network_get_network_state (backend->priv->network);
-	if (state == PK_NETWORK_ENUM_ONLINE ||
-	    state == PK_NETWORK_ENUM_MOBILE ||
-	    state == PK_NETWORK_ENUM_WIFI ||
-	    state == PK_NETWORK_ENUM_WIRED)
-		return TRUE;
-	return FALSE;
+	GNetworkMonitor *network_monitor;
+	network_monitor = g_network_monitor_get_default ();
+	return g_network_monitor_get_network_available (network_monitor);
+}
+
+/**
+ * pk_backend_convert_uri:
+ *
+ * Our proxy variable is typically 'username:password@server:port'
+ * but http_proxy expects 'http://username:password@server:port/'
+ **/
+gchar *
+pk_backend_convert_uri (const gchar *proxy)
+{
+	GString *string;
+	string = g_string_new (proxy);
+
+	/* if we didn't specify a prefix, add a default one */
+	if (!g_str_has_prefix (proxy, "http://") &&
+	    !g_str_has_prefix (proxy, "https://") &&
+	    !g_str_has_prefix (proxy, "ftp://")) {
+		g_string_prepend (string, "http://");
+	}
+
+	/* if we didn't specify a trailing slash, add one */
+	if (!g_str_has_suffix (proxy, "/"))
+		g_string_append_c (string, '/');
+
+	return g_string_free (string, FALSE);
 }
 
 /**
@@ -777,6 +954,7 @@ pk_backend_get_name (PkBackend *backend)
 {
 	g_return_val_if_fail (PK_IS_BACKEND (backend), NULL);
 	g_return_val_if_fail (backend->priv->loaded, NULL);
+	g_return_val_if_fail (pk_is_thread_default (), NULL);
 	return backend->priv->name;
 }
 
@@ -789,6 +967,7 @@ pk_backend_get_description (PkBackend *backend)
 	g_return_val_if_fail (PK_IS_BACKEND (backend), NULL);
 	g_return_val_if_fail (backend->priv->desc != NULL, NULL);
 	g_return_val_if_fail (backend->priv->loaded, NULL);
+	g_return_val_if_fail (pk_is_thread_default (), NULL);
 	return backend->priv->desc->description;
 }
 
@@ -801,6 +980,7 @@ pk_backend_get_author (PkBackend *backend)
 	g_return_val_if_fail (PK_IS_BACKEND (backend), NULL);
 	g_return_val_if_fail (backend->priv->desc != NULL, NULL);
 	g_return_val_if_fail (backend->priv->loaded, NULL);
+	g_return_val_if_fail (pk_is_thread_default (), NULL);
 	return backend->priv->desc->author;
 }
 
@@ -814,6 +994,7 @@ pk_backend_accept_eula (PkBackend *backend, const gchar *eula_id)
 
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (eula_id != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	present = g_hash_table_lookup (backend->priv->eulas, eula_id);
 	if (present != NULL) {
@@ -833,6 +1014,7 @@ pk_backend_is_eula_valid (PkBackend *backend, const gchar *eula_id)
 
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
 	g_return_val_if_fail (eula_id != NULL, FALSE);
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
 	present = g_hash_table_lookup (backend->priv->eulas, eula_id);
 	if (present != NULL)
@@ -847,15 +1029,15 @@ gchar *
 pk_backend_get_accepted_eula_string (PkBackend *backend)
 {
 	GString *string;
-	gchar *result = NULL;
-	GList *keys = NULL;
 	GList *l;
+	g_autoptr(GList) keys = NULL;
 
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
 	/* optimise for the common case */
 	if (g_hash_table_size (backend->priv->eulas) == 0)
-		goto out;
+		return NULL;
 
 	/* create a string of the accepted EULAs */
 	string = g_string_new ("");
@@ -865,10 +1047,29 @@ pk_backend_get_accepted_eula_string (PkBackend *backend)
 
 	/* remove the trailing ';' */
 	g_string_set_size (string, string->len -1);
-	result = g_string_free (string, FALSE);
-out:
-	g_list_free (keys);
-	return result;
+	return g_string_free (string, FALSE);
+}
+
+/**
+ * pk_backend_get_user_data:
+ *
+ * Return value: (transfer none): Job user data
+ **/
+gpointer
+pk_backend_get_user_data (PkBackend *backend)
+{
+	g_return_val_if_fail (PK_IS_BACKEND (backend), NULL);
+	return backend->priv->user_data;
+}
+
+/**
+ * pk_backend_set_user_data:
+ **/
+void
+pk_backend_set_user_data (PkBackend *backend, gpointer user_data)
+{
+	g_return_if_fail (PK_IS_BACKEND (backend));
+	backend->priv->user_data = user_data;
 }
 
 /**
@@ -896,17 +1097,17 @@ pk_backend_watch_file (PkBackend *backend,
 		       PkBackendFileChanged func,
 		       gpointer data)
 {
-	gboolean ret = FALSE;
-	GError *error = NULL;
-	GFile *file = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GFile) file = NULL;
 
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
 	g_return_val_if_fail (filename != NULL, FALSE);
 	g_return_val_if_fail (func != NULL, FALSE);
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
 	if (backend->priv->file_changed_func != NULL) {
 		g_warning ("already set");
-		goto out;
+		return FALSE;
 	}
 
 	/* monitor config files for changes */
@@ -917,22 +1118,16 @@ pk_backend_watch_file (PkBackend *backend,
 						      &error);
 	if (backend->priv->monitor == NULL) {
 		g_warning ("Failed to set watch on %s: %s",
-			   filename,
-			   error->message);
-		g_error_free (error);
-		goto out;
+			   filename, error->message);
+		return FALSE;
 	}
 
 	/* success */
-	ret = TRUE;
 	g_signal_connect (backend->priv->monitor, "changed",
 			  G_CALLBACK (pk_backend_file_monitor_changed_cb), backend);
 	backend->priv->file_changed_func = func;
 	backend->priv->file_changed_data = data;
-out:
-	if (file != NULL)
-		g_object_unref (file);
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -947,17 +1142,22 @@ pk_backend_finalize (GObject *object)
 
 	g_free (backend->priv->name);
 
-	g_object_unref (backend->priv->network);
 	g_key_file_unref (backend->priv->conf);
 	g_hash_table_destroy (backend->priv->eulas);
 
 	g_mutex_clear (&backend->priv->thread_hash_mutex);
 	g_hash_table_unref (backend->priv->thread_hash);
+	g_free (backend->priv->desc);
 
 	if (backend->priv->monitor != NULL)
 		g_object_unref (backend->priv->monitor);
+	if (backend->priv->transaction_inhibit_end_idle_id > 0)
+		g_source_remove (backend->priv->transaction_inhibit_end_idle_id);
+	if (backend->priv->updates_changed_id != 0)
+		g_source_remove (backend->priv->updates_changed_id);
 	if (backend->priv->handle != NULL)
 		g_module_close (backend->priv->handle);
+
 	G_OBJECT_CLASS (pk_backend_parent_class)->finalize (object);
 }
 
@@ -969,6 +1169,18 @@ pk_backend_class_init (PkBackendClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	object_class->finalize = pk_backend_finalize;
+
+	signals [SIGNAL_REPO_LIST_CHANGED] =
+		g_signal_new ("repo-list-changed",
+			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
+			      0, NULL, NULL, g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+	signals [SIGNAL_UPDATES_CHANGED] =
+		g_signal_new ("updates-changed",
+			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
+			      0, NULL, NULL, g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+
 	g_type_class_add_private (klass, sizeof (PkBackendPrivate));
 }
 
@@ -978,7 +1190,16 @@ pk_backend_class_init (PkBackendClass *klass)
 void
 pk_backend_cancel (PkBackend *backend, PkBackendJob *job)
 {
+	GCancellable *cancellable;
 	g_return_if_fail (PK_IS_BACKEND (backend));
+	g_return_if_fail (PK_IS_BACKEND_JOB (job));
+	g_return_if_fail (pk_is_thread_default ());
+
+	/* cancel */
+	cancellable = pk_backend_job_get_cancellable (job);
+	if (g_cancellable_is_cancelled (cancellable))
+		return;
+	g_cancellable_cancel (cancellable);
 
 	/* call into the backend */
 	backend->priv->desc->cancel (backend, job);
@@ -995,6 +1216,7 @@ pk_backend_download_packages (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->download_packages != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1014,6 +1236,7 @@ pk_backend_get_categories (PkBackend *backend, PkBackendJob *job)
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->get_categories != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1034,6 +1257,7 @@ pk_backend_depends_on (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->depends_on != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1056,6 +1280,7 @@ pk_backend_get_details (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->get_details != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1076,6 +1301,7 @@ pk_backend_get_details_local (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->get_details != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1096,6 +1322,7 @@ pk_backend_get_files_local (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->get_details != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1114,6 +1341,7 @@ pk_backend_get_distro_upgrades (PkBackend *backend, PkBackendJob *job)
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->get_distro_upgrades != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1132,6 +1360,7 @@ pk_backend_get_files (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->get_files != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1154,6 +1383,7 @@ pk_backend_required_by (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->required_by != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1176,6 +1406,7 @@ pk_backend_get_update_detail (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->get_update_detail != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1196,6 +1427,7 @@ pk_backend_get_updates (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->get_updates != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1217,6 +1449,7 @@ pk_backend_install_packages (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->install_packages != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1241,6 +1474,7 @@ pk_backend_install_signature (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->install_signature != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1263,6 +1497,7 @@ pk_backend_install_files (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->install_files != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1283,6 +1518,7 @@ pk_backend_refresh_cache (PkBackend *backend, PkBackendJob *job, gboolean force)
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->refresh_cache != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1306,6 +1542,7 @@ pk_backend_remove_packages (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->remove_packages != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1335,6 +1572,7 @@ pk_backend_resolve (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->resolve != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1357,6 +1595,7 @@ pk_backend_search_details (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->search_details != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1379,6 +1618,7 @@ pk_backend_search_files (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->search_files != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1401,6 +1641,7 @@ pk_backend_search_groups (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->search_groups != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1423,6 +1664,7 @@ pk_backend_search_names (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->search_names != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1442,6 +1684,7 @@ pk_backend_update_packages (PkBackend *backend, PkBackendJob *job, PkBitfield tr
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->update_packages != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1462,6 +1705,7 @@ pk_backend_get_repo_list (PkBackend *backend, PkBackendJob *job, PkBitfield filt
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->get_repo_list != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1480,6 +1724,7 @@ pk_backend_repo_enable (PkBackend *backend, PkBackendJob *job, const gchar *repo
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->repo_enable != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1499,6 +1744,7 @@ pk_backend_repo_set_data (PkBackend *backend, PkBackendJob *job, const gchar *re
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->repo_set_data != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1523,6 +1769,7 @@ pk_backend_repo_remove (PkBackend *backend,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->repo_remove != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1548,6 +1795,7 @@ pk_backend_what_provides (PkBackend *backend, PkBackendJob *job,
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->what_provides != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1567,6 +1815,7 @@ pk_backend_get_packages (PkBackend *backend, PkBackendJob *job, PkBitfield filte
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->get_packages != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1577,6 +1826,35 @@ pk_backend_get_packages (PkBackend *backend, PkBackendJob *job, PkBitfield filte
 	backend->priv->desc->get_packages (backend, job, filters);
 }
 
+/**
+ * pk_backend_upgrade_system:
+ */
+void
+pk_backend_upgrade_system (PkBackend *backend,
+			   PkBackendJob *job,
+			   PkBitfield transaction_flags,
+			   const gchar *distro_id,
+			   PkUpgradeKindEnum upgrade_kind)
+{
+	g_return_if_fail (PK_IS_BACKEND (backend));
+	g_return_if_fail (backend->priv->desc->upgrade_system != NULL);
+
+	/* final pre-flight checks */
+	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
+
+	pk_backend_job_set_role (job, PK_ROLE_ENUM_UPGRADE_SYSTEM);
+	pk_backend_job_set_transaction_flags (job, transaction_flags);
+	pk_backend_job_set_parameters (job, g_variant_new ("(tsu)",
+							   transaction_flags,
+							   distro_id,
+							   upgrade_kind));
+	backend->priv->desc->upgrade_system (backend,
+					     job,
+					     transaction_flags,
+					     distro_id,
+					     upgrade_kind);
+}
+
 /**
  * pk_backend_repair_system:
  */
@@ -1585,6 +1863,7 @@ pk_backend_repair_system (PkBackend *backend, PkBackendJob *job, PkBitfield tran
 {
 	g_return_if_fail (PK_IS_BACKEND (backend));
 	g_return_if_fail (backend->priv->desc->repair_system != NULL);
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* final pre-flight checks */
 	g_assert (pk_backend_job_get_vfunc_enabled (job, PK_BACKEND_SIGNAL_FINISHED));
@@ -1603,7 +1882,6 @@ static void
 pk_backend_init (PkBackend *backend)
 {
 	backend->priv = PK_BACKEND_GET_PRIVATE (backend);
-	backend->priv->network = pk_network_new ();
 	backend->priv->eulas = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	backend->priv->thread_hash = g_hash_table_new_full (g_direct_hash,
 							    g_direct_equal,
